function eflag = ComputeBootstrap(ParameterFile,task1,task2,reps)
% Compute bootstrap replications for elasticities, costs, tax results and welfare.
%
% ParameterFile  (string)  File containing parameters that define model
% task1          (integer) number of first replication to compute
% task2          (integer) number for last replication to compute
% reps           (integer) total number of replications to compute
%
%                          1 <= task1 <= task2 <= reps
%
% Revision history
% 2017MAR21  Cleaned version for publication.

% Set path.
setpath;

% Set scale for printing computation time.
TimeFactor = 3600; % 3600 = timings measured in hours
                   % 60   = timings measured in minutes
                   
% 1) load data
% 2) load base parameters and covariance matrix of parameters
% 3) draw a parameter vector.
% 4) compute:
%    a) elas(j,j) for all j
%    b) cost(j)   for all j
%    c) for each tax setting:
%       i) price(j),share(i,j),CV(i,j)
%

% Read parameter values from file
eval(ParameterFile)

% create bootstrap output directory
BootstrapDir = fullfile(OutDir,'bootstrap');
if exist(BootstrapDir,'dir')==0
  system(['mkdir ',BootstrapDir]);    
end

% initialize random stream
rng('default');

% size(DeltaX)      = (J*N x K)
% size(DeltaZ.z(i)) = (J*N x nQuadAll)

StartTime = clock;
timings = zeros(reps,2);

% Load results from estimation
load(EstimationResultsMatFile);
SigX = inv(Hess2);
CX   = chol(SigX,'lower');

% nTax:  number of tax experiments
nTax = 5;

% TaxResults0 = [firm, product, sats, market, month, region, p0]
% TaxResults(i1,:,t1,b1) = [cost,s0,p1,s1,p2_firm,s2]
% p0 = base price
% s0 = base share (predicted)
% p1 = price with tax (100% passthrough)
% p2_firm = firm price in Nash equilibrium with tax
% s2      = share in Nash equilibrium with tax
TaxResults = zeros(noptions*nMarkets,6,nTax,reps);

nprocs = reps;
pid =task1;

% treatments to compute
% 5 of them
TaxTreatment   = [1 2 3 1 2]';
OwnerTreatment = [0 0 1 1 1]';
SampleSize = 1000;
BigTax = 0;
CV         = zeros(SampleSize,nTax,reps);
cost       = zeros(noptions,nMarkets,reps);
HHElas     = zeros(SampleSize,noptions,reps);
HHShare    = zeros(SampleSize,noptions,reps);

% Select random subsample for calculations
e1     = rand(nHH,1);
e1sort = sort(e1);
iSample = e1<=e1sort(SampleSize);
tSample = rand(nHH,1);  % there are nT observations per HH
                      % this is used to select one random obs. per HH

for b1=1:task2
  tic  
  xFreeBoot = xFree2 + CX*randn(iFree2.n,1);
  if (b1>=task1)
    % Compute elasticities and costs for current bootstrap replication
    [TempElas,cost(:,:,b1),TempShare,HHIncome] = ...
        AnalyzeResults(ParameterFile,tSample,SampleSize,iSample, ...
                       BootstrapDir,b1,xFreeBoot);
    for i1=1:noptions
      HHElas(:,i1,b1)=reshape(TempElas(i1,i1,:),[SampleSize 1]);
    end
    HHShare(:,:,b1) = TempShare';
    save(fullfile(BootstrapDir,['boot_A',int2str(task1),'.mat']), ...
         'HHElas','HHShare','cost','HHIncome');
    % Compute tax results for current bootstrap replication 
    for t1=1:5
      ComputeTaxResults(nprocs,pid,ParameterFile, ...
                        TaxTreatment(t1),OwnerTreatment(t1),SampleSize,iSample, ...
                        BigTax,BootstrapDir,b1,xFreeBoot);
      [TempTaxResults,CV(:,t1,b1)] =                        ...
         SaveTaxResults(ParameterFile,TaxTreatment(t1), ...
                        OwnerTreatment(t1),SampleSize,iSample,BigTax,BootstrapDir,b1,xFreeBoot);
      TaxResults0 = TempTaxResults(:,[(1:6)';8]);
      TaxResults(:,:,t1,b1) = TempTaxResults(:,[7;(9:13)']);
      save(fullfile(BootstrapDir,['boot_B',int2str(task1),'.mat']), ...
           'TaxResults0','TaxResults','CV');
    end  
  end % if (b1>=cpu_tasks(cpu_id,1) ...
 
  timings(b1,1) = toc/TimeFactor;
  if b1==1
    timings(b1,2) = timings(b1,1);
  else
    timings(b1,2) = timings(b1-1,2)+timings(b1,1);
  end
  display('b1 reps StartTime timing');
  display([b1 reps StartTime(4:5) timings(b1,:)]);
end
eflag=0;