function ComputeTaxResults(nProcessors,pid,ParameterFile,TaxType, ...
                           OwnershipFlag,SampleSize,iSample,BigTax,BootstrapDir,brep,xFreeBoot)
% nProcessors   = number of processors
% pid           = process id of current process
% ParameterFile = name of parameter file
% TaxType       = 1 excise, 2 ad valorem, 3 no tax
% OwnershipFlag = 0 baseline, 1 single product firms
% SampleSize    = sample size used for tax computations
% iSample
% BigTax        = flag for big tax experiment
% BootstrapDir  = directory to store bootstrap results
% brep          = index for current bootstrap replicatoin
% xFreeBoot     = vector of parameters for current bootstrap replication

% Compute tax results
% 
% Revision history: 
% 2017MAR18  Cleaned version for publication.

%--------------------------------------------------------------------------
%% INITIALISE:  Section 1
%--------------------------------------------------------------------------
if nargin<8
  BigTax = 0;
end
if nargin<9
    brep=0;
end
eval(ParameterFile)

ParallelFlag=0;
if ParallelFlag==1
  v=version('-release');                     
  if strncmp(v,'2011',4) || strncmp(v,'2012',4)                         
    pp =matlabpool('size');
    if pp>0
      matlabpool('close','force','local');
    end
    matlabpool('open');
  else
    if ~isempty(gcp('nocreate'))
      delete(gcp('nocreate'));
    end
    parpool;
  end
end

%--------------------------------------------------------------------------
%% Parameters
%--------------------------------------------------------------------------
if nargin>8
  [OutDir,BaseTaxDataFile,CostDataFile] = ...
      SetBootstrapFilenames(BootstrapDir,brep);
end

% Tax parameters
%   Tax.Type:       1 Excise tax
%                   2 Ad Valorem tax
%                   3 No tax
%   OwnershipFlag:  0 Baseline ownership structure
%         p          1 Single product firm structure
if OwnershipFlag==1
  Tax.NewTaxFlag=0;   % 0 = use same tax rate for all ownership structures
                      % 1 = different tax rate for each ownership structure
end
Tax.Type = TaxType;

Tax = SetTaxParameters(Tax,OwnershipFlag,OutDir,noptions);
if Tax.Type==1
  Tax.tax = ComputeTax2HitTarget(ParameterFile,Tax,OwnershipFlag,BaseTaxDataFile);
  if BigTax==1
    Tax.tax=10*Tax.tax;
  end
elseif Tax.Type==2
  Tax.tau = ComputeTax2HitTarget(ParameterFile,Tax,OwnershipFlag,BaseTaxDataFile);
  if BigTax==1
    Tax.tau=10*Tax.tau;
  end
end

if OwnershipFlag==1 && Tax.Type<3
  % use prices from output of (OwnershipFlag==1 && Tax.Type==3)
  % use costs from datafile
  PriceOption   = 3;
  PriceDataFile = Tax.PriceDataFile;
end

[Subset,TempDirList] = ComputePartition(nProcessors,nMarkets,OutDir);

%--------------------------------------------------------------------------
%% Load and extract data
%--------------------------------------------------------------------------
%   Ownership Matrix
%   ProductCodeRaw
%   SortedData
%   (xFree2,iFree2,beta2,C)
%   nFirms,AllFirmsProducts,NewHHProductCode
%   Nodes,Weights,nQuadAll
%
% BaseTaxDataFile is created by AnalyseResults
aa=clock;
disp(['Loading ',BaseTaxDataFile,'. Current time: ', ...
      int2str(aa(4)),':',int2str(aa(5))]);
tic
load(BaseTaxDataFile);

% set parameter values to values from current bootstrap iteration
if nargin>8
  xFree2 = xFreeBoot;
end

% Select Sample of data
if SampleSize>0 && nHH>SampleSize
  nHH_full = nHH;
  nHH = SampleSize;
  if ((SparseGridFlag==2 || SparseGridFlag==3) && MCDrawFlag==2)
    % Monte carlo integration nodes
    % different draws for each HH
    % Nodes = (nHH*nQuadAll,RCDim)
    Nodes = reshape(Nodes,[nHH_full nQuadAll RCDim]);
    Nodes = reshape(Nodes(iSample,:,:),[nHH*nQuadAll RCDim]);
  end
  i2 = reshape(repmat(iSample',[noptions 1]),[nHH*noptions 1]);
  SortedData       = SortedData(i2,:);
  SortedHHFirmCode = SortedHHFirmCode(i2,:);
  NewHHProductCode = NewHHProductCode(i2,:);
  basis.EquivalenceScale = basis.EquivalenceScale(i2);
end

[X,Z]=NewData(SortedData,SortedData(:,RawColumn.FixedEffect), ...
              RawColumn,basis,ZIndex,nHH,noptions);
X.X=full(X.X);
aa=clock;
t1=toc/60;
disp(['Current time: ',int2str(aa(4)),':', ...
      int2str(aa(5)),'. Data load time: ',int2str(t1)]);

% Variables used to gross up from data to actual market shares
if ~isempty(RawColumn.SampleWeight)
  HHWeights = reshape(SortedData(:,RawColumn.SampleWeight),noptions,nHH);
else
  HHWeights = ones(noptions,nHH);
end
RawScale = sum(HHWeights(1,:));
NewScale = nHH;
HHWeights = NewScale*HHWeights/RawScale;

% Data on saturated fat content  (first noptions rows only)
Sats=SortedData(1:noptions,RawColumn.Sats);

%--------------------------------------------------------------------------
%% Define market.
% In data, 
%     price = (actual price)               when Choice==1
%     price = (avg. price in region,month) when Choice==0
%
%--------------------------------------------------------------------------
[Market,nHH] = DefineMarket(SortedData,RawColumn,MarketDefinition, ...
                            noptions,nMonths,nRegions);

%--------------------------------------------------------------------------
%% Set baseline price  
%  PriceOption == 1:  use weighted average of prices in data
%  PriceOption == 2:  use data from PriceDataFile
%--------------------------------------------------------------------------
%if TaxType<3 && OwnershipFlag==1
  % base price is set based on counterfactual prices, not real prices
%  PriceOption=2;
%end
MarketShareHandle = ...
      @(X,Z)MarketShare(X,Z, ...
                        xFree2,iFree2,beta2,CDiag,COffDiag, ...
                        LogNormalFlag,LogNormalIndex,       ...
                        EstColumn,ZIndex,        ...
                        noptions,nHH,RCDim,RCRank, ...
                        nFirms,AllFirmsProducts, ...
                        NewHHProductCode, ...
                        Nodes,Weights,nQuadAll);

[HHPrice,BasePrice,X,Z] = ...
    ComputeBasePrice(PriceOption,SortedData,RawColumn,           ...
                     MarketShareHandle,X,Z,Market,               ...
                     basis,EstColumn,ZIndex,                     ...
                     noptions,RCDim,nMarkets,nRegions,  ...
                     SortedProductCodeRaw,OutsideOptionColumn,   ...
                     PriceDataFile,MarketDefinition);
% Save NewX to TempDir
save(fullfile(TempDirList{pid},'X.mat'), ...
     'X','Z',                       ...
     '-v7.3');

%--------------------------------------------------------------------------
%% Calculate BASELINE shares, gradient of shares, and COSTS
%--------------------------------------------------------------------------
[HHShare,HHGradS]=MarketShareHandle(X,Z);

if PriceOption<3
  if basis.ExpendFlag==0
    % utility depends on -f(p)
    t1=1;
  else
    % utility depends on f(y-p): multiply HHGradS by negative one
    t1=-1;
  end
  [BaseCost,HHCost]=ComputeCosts(HHShare,t1*HHGradS,HHWeights,BasePrice,  ...
                                 noptions,nHH,nFirms,AllFirmsProducts, ...
                                 Market(SortedData(:,RawColumn.Choice)==1), ...
                                 nMarkets,OutsideOptionColumn);
  clear HHGradS;                                 
  save(CostDataFile,'BaseCost','HHCost');                             
elseif PriceOption==3
  % Baseline prices are counterfactual
  % load baseline costs from file.
  load(CostDataFile); 
end

%--------------------------------------------------------------------------
%% New ownership matrix
% OwnershipFlag=0    True ownership
% OwnershipFlag=1    New ownership structure
%
% Ownership structure defined by:
%    AllFirmsProducts
%    AllCompetitorProducts
%    SortedHHFirmCode
%    SortedFirmCode
%    nFirms
%  
if OwnershipFlag==1                 
  % Create new ownership structure in which each product is a single firm
  [nFirms,AllFirmsProducts,AllCompetitorProducts] ...
      = CreateNewOwnership(noptions);
end

%--------------------------------------------------------------------------
%% Calculate prices and shares with tax:    NO FIRM RESPONSE
%--------------------------------------------------------------------------
%[HHTaxPrice,HHTaxShare,TaxPrice] ...
%        = ComputeTaxShare(Tax,xFree2,iFree2,beta2,CDiag,COffDiag,       ... 
%                          NewX,NewZ,GradNewX,                       ...
%                          EstColumn,       ...
%                          noptions,nHH,RCDim,RCRank,                ...
%                          nFirms,AllFirmsProducts,NewHHProductCode, ...
%                          Nodes,Weights,nQuadAll,                   ...
%                          BasePrice,HHPrice,Sats,nMarkets);

%--------------------------------------------------------------------------
%% Calculate Nash Equilibrium
%
% Set up matrices to store results
clear SortedData SortedHHFirmCode HHPrice HHCost;
[NashExitFlag,NashOutput,TotalTime] = ComputeEquilibrium( ...
    HHWeights,X,Z,Market,NewHHProductCode,                ...    
    BasePrice,BaseCost,Sats,AllFirmsProducts,AllCompetitorProducts, ...
    nMarkets,noptions,nHH,RCDim,RCRank,nFirms,Tax,                  ...
    IncludeOutsideOption,OutsideOptionColumn,                       ...
    LogNormalFlag,LogNormalIndex,basis,EstColumn,ZIndex,            ...
    xFree2,iFree2,beta2,CDiag,COffDiag,                             ...
    Nodes,Weights,nQuadAll, ...
    Subset,pid,TempDirList,OwnershipFlag,MultipleEquilibriumFlag,brep);
