function [X,Z] = NewData(data,BrandDummies,RawColumn,basis, ...
                         ZIndex,N,J)
% Compute new data matrix using new prices for each product
% data         = (J*N x k1) data matrix
% BrandDummies = (J*N x k2)
% RawColumn    = (structure)  list contents of each column in data 
% basis        = (structure)  details of structure of basis functions f(p)
% ZIndex       = (structure)  list of contents of Z
% RCDim        = (1 x 1)      dimension of random coefficients
% N            = (1 x 1)      number of households
% J            = (1 x 1)      number of options in choice set
% nQuadAll     = (1 x 1)      number of quadrature nodes
%
% Revision history
% 2017MAR19 Cleaned version for publication.
%
% Extract X regressors from data
% f   = (J*N x nf)  f(:,i1) = B(p,i1) where B(:,i1) is the i1'th basis function
% df  = (J*N x nf)  gradient of f w.r.t. p 
% d2f = (J*N x nf)  hessian of f w.r.t. p
if basis.flag<=6
  expenditure = data(:,RawColumn.Expenditure);
  iZero    = (data(:,RawColumn.Price)==0);
    
  [f,df,d2f]=ComputeBasisFunctions(expenditure,basis);
  f=f';
  df=df';
  d2f = d2f';
  if basis.ZeroPriceFlag==1
    f(iZero,:)   = 0;
    df(iZero,:)  = 0;
    d2f(iZero,:) = 0;
  end
  nx = size(RawColumn.Interaction,1); 

  % X.X = [x1.*f(p) x2 BrandDummies ]
  x1 = zeros(N*J,(nx+1)*basis.nf);
  X.GradX = x1;
  X.HessX = x1;
  X.xInteraction     = data(:,RawColumn.Interaction);
  X.Price            = sparse(data(:,RawColumn.Price));
  X.GrossExpenditure = sparse(expenditure) + X.Price ;
  
  for i1=0:nx
  for i2=1:basis.nf
      index = i1*basis.nf+i2;
    if i1==0
      x1(:,index) = f(:,i2);
      X.GradX(:,index) = df(:,i2);
      X.HessX(:,index) = d2f(:,i2);
    else
      x1(:,index) = f(:,i2).*X.xInteraction(:,i1);
      X.GradX(:,index) = df(:,i2).*X.xInteraction(:,i1);
      X.HessX(:,index) = d2f(:,i2).*X.xInteraction(:,i1);
    end
  end
  end
  X.X = [x1, data(:,RawColumn.Regressors), BrandDummies];
  X.X=sparse(X.X);
  X.GradX = sparse(X.GradX);
  X.HessX = sparse(X.HessX);
  X.xInteraction = sparse(X.xInteraction);

  % update Z
  % Z.Z:   Data to be interacted with random coefficients
  % size(Z.Z)           = (N*J x RCDim)
  % Z.Z(:,ZIndex.iP)    = elements of TempZ that depend on f(price)
  % Z.Z(:,ZIndex.iNonP) = elements of TempZ that do not depend on price
  % f(:,Zindex.iF)        = columns of f that are interacted with random coefficients
  % data(:,ZIndex.RawColumn) = elements of data to interact with random coefficients
  if nargout>1
    Z = [];  
    Z.Z=[f(:,ZIndex.iF) data(:,ZIndex.RawColumn)];
    Z.GradZ = df(:,ZIndex.iF);
    Z.HessZ = d2f(:,ZIndex.iF);
  end     % if nargout>1

end  % if basis.flag<=4
