function [TaxResults,WelfareResults] = ...
         SaveTaxResults(ParameterFile,TaxType,OwnershipFlag,SampleSize,iSample,BigTax, ...
                        BootstrapDir,brep,xFreeBoot)
% Compute tax results
% 
% Revision history:
% 2017MAR19  Cleaned version for publication.

%--------------------------------------------------------------------------
%% INITIALISE:  Section 1
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Parameters
%--------------------------------------------------------------------------
nProcessors=1;
eval(ParameterFile);

if nargin>6
  [OutDir,BaseTaxDataFile,CostDataFile] = ...
      SetBootstrapFilenames(BootstrapDir,brep);
  % setup new TempDir for bootstrap results if this is a bootstrap
  % replication.
  TempDir = fullfile(OutDir,'temp');
  if exist(TempDir,'dir')==0
    system(['mkdir ',TempDir]);
  end
end

% Tax parameters
%   Tax.Type:       1 Excise tax
%                   2 Ad Valorem tax
%                   3 No tax
%   OwnershipFlag:  0 Baseline ownership structure
%                   1 Single product firm structure
Tax.Type      = TaxType;
Tax = SetTaxParameters(Tax,OwnershipFlag,OutDir,noptions);
if OwnershipFlag==1
  Tax.NewTaxFlag=0;   % 0 = use same tax rate for all ownership structures
                      % 1 = different tax rate for each ownership structure
end

if Tax.Type==1
  Tax.tax = ComputeTax2HitTarget(ParameterFile,Tax,OwnershipFlag,BaseTaxDataFile);
  if BigTax==1
    Tax.tax=10*Tax.tax;
  end
elseif Tax.Type==2
  Tax.tau = ComputeTax2HitTarget(ParameterFile,Tax,OwnershipFlag,BaseTaxDataFile);
  if BigTax==1
    Tax.tau=10*Tax.tau;
  end
end

if OwnershipFlag==1 && Tax.Type<3
  % use prices from output of (OwnershipFlag==1 && Tax.Type==3)
  % use costs from datafile
  PriceOption=3;  
  PriceDataFile = Tax.PriceDataFile;
end

[Subset,TempDirList] = ComputePartition(nProcessors,nMarkets,OutDir);

%--------------------------------------------------------------------------
%% Load and extract data
%--------------------------------------------------------------------------
%   Ownership Matrix
%   ProductCodeRaw
%   SortedData
%   (xFree2,iFree2,beta2,C)
%   nFirms,AllFirmsProducts,NewHHProductCode
%   Nodes,Weights,nQuadAll
%
% BaseTaxDataFile is created by AnalyseResults
aa=clock;
disp(['Loading ',BaseTaxDataFile,'. Current time: ', ...
      int2str(aa(4)),':',int2str(aa(5))]);
tic
load(BaseTaxDataFile);

% set parameter values equal to current values from bootstrap
if nargin>6
  xFree2 = xFreeBoot;
end

if SampleSize>0 && nHH>SampleSize
  nHH = SampleSize;
  i2 = reshape(repmat(iSample',[noptions 1]),[nHH*noptions 1]);
  SortedData       = SortedData(i2,:);
  SortedHHFirmCode = SortedHHFirmCode(i2,:);
  NewHHProductCode = NewHHProductCode(i2,:);
end

[X,Z]=NewData(SortedData,SortedData(:,RawColumn.FixedEffect), ...
              RawColumn,basis,ZIndex,nHH,noptions);
X.X=full(X.X);
aa=clock;
t1=toc/60;
disp(['Current time: ',int2str(aa(4)),':', ...
      int2str(aa(5)),'. Data load time: ',int2str(t1)]);

% Variables used to gross up from data to actual market shares
if ~isempty(RawColumn.SampleWeight)
  HHWeights = reshape(SortedData(:,RawColumn.SampleWeight),noptions,nHH);
else
  HHWeights = ones(noptions,nHH);
end
RawScale = sum(HHWeights(1,:));
NewScale = nHH;
HHWeights = NewScale*HHWeights/RawScale;

% Data on saturated fat content  (first noptions rows only)
Sats=SortedData(1:noptions,RawColumn.Sats);

%--------------------------------------------------------------------------
%% Define market.
% In data, 
%     price = (actual price)               when Choice==1
%     price = (avg. price in region,month) when Choice==0
%
%--------------------------------------------------------------------------
[Market,nHH] = DefineMarket(SortedData,RawColumn,MarketDefinition, ...
                            noptions,nMonths,nRegions);

%--------------------------------------------------------------------------
%% Set baseline price  
%  PriceOption == 1:  use weighted average of prices in data
%  PriceOption == 2:  use data from PriceDataFile
%--------------------------------------------------------------------------
MarketShareHandle = ...
      @(X,Z)MarketShare(X,Z, ...
                        xFree2,iFree2,beta2,CDiag,COffDiag, ...
                        LogNormalFlag,LogNormalIndex,       ...
                        EstColumn,ZIndex,        ...
                        noptions,nHH,RCDim,RCRank, ...
                        nFirms,AllFirmsProducts, ...
                        NewHHProductCode, ...
                        Nodes,Weights,nQuadAll);

[HHPrice,BasePrice,X,Z] = ...
    ComputeBasePrice(PriceOption,SortedData,RawColumn, ...
                     MarketShareHandle,X,Z,Market,      ...
                     basis,EstColumn,ZIndex,            ...
                     noptions,RCDim,nMarkets,nRegions, ...
                     SortedProductCodeRaw,OutsideOptionColumn, ...
                     PriceDataFile,MarketDefinition);

% Save X to TempDir
save(fullfile(TempDir,'X.mat'), ...
     'X','Z',   ...
     '-v7.3');

%--------------------------------------------------------------------------
%% Calculate BASELINE shares, gradient of shares, and COSTS
%--------------------------------------------------------------------------
[HHShare,HHGradS] = MarketShareHandle(X,Z);
if PriceOption<3
  if basis.ExpendFlag==0
    % utility depends on -f(p)
    t1=1;
  else
    % utility depneds on f(y-p): multiply HHGradS by negative one
    t1=-1;
  end

  [BaseCost,HHCost]=ComputeCosts(HHShare,t1*HHGradS,HHWeights,BasePrice,  ...
                                 noptions,nHH,nFirms,AllFirmsProducts, ...
                                 Market(SortedData(:,RawColumn.Choice)==1), ...
                                 nMarkets,OutsideOptionColumn);
  clear HHGradS;                                 
  save(CostDataFile,'BaseCost','HHCost');                             
elseif PriceOption==3
  % Baseline prices are counterfactual
  % load baseline costs from file.
  load(CostDataFile); 
end

%--------------------------------------------------------------------------
%% New ownership matrix
% OwnershipFlag=0    True ownership
% OwnershipFlag=1    New ownership structure
%
% Ownership structure defined by:
%    AllFirmsProducts
%    AllCompetitorProducts
%    SortedHHFirmCode
%    SortedFirmCode
%    nFirms
%  
if OwnershipFlag==1                 
  % Create new ownership structure in which each product is a single firm
  [nFirms,AllFirmsProducts,AllCompetitorProducts] ...
      = CreateNewOwnership(noptions);
end

%--------------------------------------------------------------------------
%% Calculate prices and shares with tax:    NO FIRM RESPONSE
%--------------------------------------------------------------------------
[HHTaxPrice,HHTaxShare,TaxPrice] ...
        = ComputeTaxShare(Tax,xFree2,iFree2,beta2,CDiag,COffDiag,       ... 
                          X,Z,LogNormalFlag,LogNormalIndex, ...
                          basis,EstColumn,ZIndex,           ...
                          noptions,nHH,RCDim,RCRank,             ...
                          nFirms,AllFirmsProducts,NewHHProductCode, ...
                          Nodes,Weights,nQuadAll,                   ...
                          BasePrice,HHPrice,Sats,nMarkets);
HHTaxExpenditure = X.GrossExpenditure - HHTaxPrice;

%--------------------------------------------------------------------------
%% Merge results
[NashProducerPrice,Residual,HHNashConsumerPrice] = ...
          MergeResults(Market,Sats,Tax, ...
                       IncludeOutsideOption,OutsideOptionColumn, ...
                       noptions,nHH,nMarkets,                    ...
                       nProcessors,Subset,TempDirList,OwnershipFlag);
HHNashExpenditure = X.GrossExpenditure - HHNashConsumerPrice;                   

% Compute predict market shares in new equilibrium
% Put HHTaxPrice data into (NewX,NewZ) data matrices
iNonZero = HHPrice~=0;
[X,Z]=UpdatePriceTerms(HHNashExpenditure,X,Z,basis,EstColumn,ZIndex, ...
                       iNonZero,RCDim);

HHNashShare=MarketShareHandle(X,Z);                    
                    
% Save HHNashShare in product level format
[BaseShare,TaxShare,NashShare]=ExtractProductLevelResults( ...
          Market,nMarkets,noptions, ...
          HHShare,HHTaxShare,HHNashShare,HHWeights);

% Load (HH,product) results into matrix:
%   HHTaxResults
% Column   Variable
% 1        HHID
% 2        Firm
% 3        ProductCode
% 4        market
% 5        month
% 6        region
% 7        cost
% 8        sats
% 9        BasePrice
% 10       BaseShare
% 11       TaxPrice
% 12       TaxShare
% 13       NashConsumerPrice
% 14       NashShare
% 15       HHWeights

HHTaxResults=[SortedData(:,RawColumn.HH),     ...
              SortedHHFirmCode,               ...
              SortedData(:,RawColumn.ProductCode), ...
              Market,                         ...
              SortedData(:,RawColumn.Month),  ...
              SortedData(:,RawColumn.Region), ...
              HHCost(:),                      ...
              SortedData(:,RawColumn.Sats),   ...
              HHPrice,                        ...
              HHShare(:),                     ...
              HHTaxPrice,                     ...
              HHTaxShare(:),                  ...
              HHNashConsumerPrice,            ...
              HHNashShare(:),                 ...
              HHWeights(:)];

% Save HHTaxResults to .mat file   (matlab format)
save(Tax.TaxResultsMatFile,'HHTaxResults');

% save HHTaxResults to .raw file  (ascii format)
eval(['save ',Tax.HHLevelTaxResultsRawFile, ...
      ' HHTaxResults -ascii -double -tabs']);

% Create TaxResults:  (matrix with results at product level 
% Column   Variable
% 1        Firm
% 2        Product  
% 3        Sats
% 4        Market 
% 5        Month
% 6        Region
% 7        Cost
% 8        BasePrice
% 9        BaseShare 
% 10       TaxPrice
% 11       TaxShare
% 12       NashProducerPrice
% 13       NashShare
% 14       Residual
if MarketDefinition==1
  Month=zeros(noptions*nMarkets,1);
  Region=zeros(noptions*nMarkets,1);
elseif MarketDefinition==2
  Month=kron((1:nMonths)',ones(noptions,1));
  Region=zeros(noptions*nMarkets,1);
elseif MarketDefinition==3
  Month=zeros(noptions*nmarkets,1);
  Region=kron((1:nRegions)',ones(noptions,1));
elseif MarketDefinition==4
  temp=kron((1:nMonths)',ones(nRegions,1));
  Month=kron(temp,ones(noptions,1));
  temp=kron(ones(nMonths,1),(1:nRegions)');
  Region=kron(temp,ones(noptions,1));
end
TaxResults = [kron(ones(nMarkets,1),SortedFirmCode),       ...
              kron(ones(nMarkets,1),SortedProductCodeRaw), ...
              kron(ones(nMarkets,1),Sats),                 ...
              kron((1:nMarkets)',ones(noptions,1)),        ...
              Month,                                       ...
              Region,                                      ...
              BaseCost(:),                                 ...
              BasePrice(:),                                ...
              BaseShare(:),                                ...
              TaxPrice(:),                                 ...
              TaxShare(:),                                 ...
              NashProducerPrice(:),                        ...
              NashShare(:),                                ...
              Residual(:)];
          
% Save TaxResults to .raw file (product level, ascii format)          
eval(['save ',Tax.ProductLevelTaxResultsRawFile, ...
      ' TaxResults -ascii -double -tabs;']);
  
if OwnershipFlag==1 && Tax.Type==3
  NewHHPrice = HHNashConsumerPrice;
  BasePrice  = NashProducerPrice;
  save(Tax.PriceDataFile,'NewHHPrice','BasePrice');
end
  
% Compute welfare results
HHID = SortedData(:,RawColumn.HH);

WelfareResults_linear = ...
    ComputeWelfare(xFree2,iFree2,beta2,CDiag,COffDiag,HHID, ... 
                   HHPrice,HHTaxPrice,HHNashConsumerPrice, ...
                   LogNormalFlag,LogNormalIndex, ...
                   basis,EstColumn,ZIndex,       ...
                   noptions,nHH,RCDim,RCRank,TempDir,   ...
                   Nodes,Weights,nQuadAll);

% Ex post CV: nonlinear prices
if nargin<=6
  brep=0;
  WelfareResults_Dagsvik = ...
      ComputeWelfare_Dagsvik(xFree2,iFree2,beta2,CDiag,COffDiag,HHID, ... 
                             HHPrice,HHTaxPrice,HHNashConsumerPrice, ...
                             LogNormalFlag,LogNormalIndex, ...
                             basis,EstColumn,ZIndex, ...
                             noptions,nHH,RCDim,RCRank,TempDir,   ...
                             Nodes,Weights,nQuadAll,brep);
else
    WelfareResults_Dagsvik = ...
      ComputeWelfare_Dagsvik(xFree2,iFree2,beta2,CDiag,COffDiag,HHID, ... 
                             HHPrice,HHTaxPrice,HHNashConsumerPrice, ...
                             LogNormalFlag,LogNormalIndex, ...
                             basis,EstColumn,ZIndex, ...
                             noptions,nHH,RCDim,RCRank,TempDir,   ...
                             Nodes,Weights,nQuadAll,brep);
end
% ex ante CV: nonlinear prices
WelfareResults_ExAnte = ...
    ComputeWelfare_ExAnte(xFree2,iFree2,beta2,CDiag,COffDiag, ....
                          HHID,HHPrice,HHTaxPrice,           ...
                          HHNashConsumerPrice,          ...
                          LogNormalFlag,LogNormalIndex, ...
                          basis,EstColumn,ZIndex,       ...
                          noptions,nHH,RCDim,RCRank, ...
                          TempDir,Nodes,Weights,nQuadAll);

% WelfareResults = [HHID CV_tax_linear CV_nash_linear, ...
%                        CV_tax_ExPost CV_nash_ExPost, ...
%                        CV_tax_ExAnte CV_nash_ExAnte]
WelfareResults = [WelfareResults_linear, ...
                  WelfareResults_Dagsvik(:,2:3), ...
                  WelfareResults_ExAnte(:,2:3)];
save(Tax.WelfareResultsFile,'WelfareResults','-ascii','-double','-tabs');

% outputs results for bootstrap
if nargin>6
  WelfareResults = WelfareResults_Dagsvik(:,3); 
end
