% Set values for all input parameters.
%
% Revision history
% 2017MAR18 LN   Final version for publication.
%                baseline model (cubic utility)
%                1) f(expend) interacted with HH_Size
%                2) pack size interacted with HH_Size
%                2) labour status
%                3) promotion
%                4) ad +  ad2

% DataDir:  Name of directory where data is stored
% OutDir:   output directory name

Path='C:/GriffithNesheimOConnell.2017.QE';
DataDir=fullfile(path,'Stata_files/Estimation');
OutDir=fullfile(path,'Matlab_files');

% BaseString for input files, output files and other uses
RawDataFile = fullfile(DataDir,'MatlabRawData.raw');
BaseString = 'V130';
OutDir     = fullfile(OutDir,'Nonlinear');

if exist(OutDir,'dir')==0
  system(['mkdir ',OutDir]);
end

% Output directory for figures
FigDir = fullfile(OutDir,'figures');
if exist(FigDir,'dir')==0
  system(['mkdir ',FigDir]);
end

% directory to hold temporary results
TempDir = fullfile(OutDir,'temp');
if exist(TempDir,'dir')==0
  system(['mkdir ',TempDir]);
end

DataHeaderLine=1;   % 1 if 1st line of data contains labels
                    % 0 if no labels

% Name of data file with data on Firms and their product ownership
FirmDataFile=fullfile(DataDir,'ownership.raw');

% PriceDataFiles
PriceDataFile=cell(4,1);
PriceDataFile{1}=fullfile(DataDir,'average_prices.raw');
PriceDataFile{2}=fullfile(DataDir,'Month_prices_1.raw');
PriceDataFile{3}=fullfile(DataDir,'regional_prices.raw');
PriceDataFile{4}=fullfile(DataDir,'Butter_Prices_Month_Region.raw');

% Cost data file
CostDataFile    =fullfile(OutDir,'costs.mat');

% Market definitions
nMonths=12;
nRegions=0;
MarketDefinition=2; % 1 = Only one market
                    % 2 = One market per month   (12 markets)
                    % 3 = One market per region  (3 markets)
                    % 4 = One market per (month,region) (12*nRegions markets)
nMarkets=12;                    
                    
% Choose baseline prices 
PriceOption=1;    % PriceOption=1:  Compute avg. prices weighted by predicted share.
                  % PriceOption=2:  Load avg. prices from datafile.  

% Name of location to save data in matlab format.
SaveDataFile       = fullfile(OutDir,[BaseString,'.mat']);
SaveDataFlag = 1;
EstimationDataFile = fullfile(OutDir,'EstimationData.mat');

% SummaryFile:  Name of file to write summary statistics
SummaryFile=fullfile(OutDir,[BaseString,'SummaryStats.tex']);

% Name of Model 1 output file
%    these hold results from estimation of logit (w/o random coefficients)
OutFile1=fullfile(OutDir,[BaseString,'_Results1.txt']);
MatFile1=fullfile(OutDir,[BaseString,'_1']);
HessFile1=fullfile(OutDir,[BaseString,'_Hess1']);

% Name of Model 2 output file
%    these hold results from estimation of random coefficients logit
OutFile2=fullfile(OutDir,[BaseString,'_Results2.txt']);
MatFile2=fullfile(OutDir,[BaseString,'_2']);
HessFile2=fullfile(OutDir,[BaseString,'_Hess2']);

% EstimationResults mat file
EstimationResultsMatFile = fullfile(OutDir,[BaseString,'_EstResults.mat']);

% Name of output file with TaxResults
%    output files for counterfactual tax results
BaseTaxDataFile=fullfile(OutDir,[BaseString,'BaseTaxData.mat']);
SortedDataFile   = fullfile(OutDir,'SortedData.mat');

% Name of output file for elasticities
ElasticityMatFile=fullfile(OutDir,[BaseString,'_Elas.mat']);
ElasticityTxtFile1=fullfile(OutDir,[BaseString,'_Elas1.txt']);
ElasticityTxtFile2=fullfile(OutDir,[BaseString,'_Elas2.txt']);

% Number of columns in dataset
%    Assumptions
%       row 1 is variable name
%       column 1 is HHID
%       column 2 row j is 1 if HHID chose store j, 0 otherwise
%       Last observation for each household contains column 2 = 1, all others
%         column 2 == 0
%       column 3 is product code
%       column 4 is brand indicator, This will be used to define fixed
%                                    effects.
% nvars = number of columns in raw data
nT       = 5;   % number of observations per household
nvars    = 129; % number of variables in raw data
noptions = 48;  % number of options in choice set

% nHH_Raw = Number of households in raw data.
% nHH     = Number of households to load and analyse
nHH_Raw = 10012;
nHH     = 10012;   % required:  nHH <= nHH_Raw

MultipleEquilibriumFlag = 0;  % 1 = test for multiple equilibrium
                              % 0 = otherwise

% raw==1    To load data from raw file (csv format)
% raw==0    To load data from matlab format file.
raw=1;

% Define f(y-p) = utility as function of (y-p) 
basis.flag   = 1;   % 1 = cubic, 2 = splines, 3 = Cheb Poly, 4 = linear
                    % 5 = log of expenditure
                    % 6 = cubic in log expenditure
basis.ZeroPriceFlag = 2;  % 1 set f(p) = 0 for p=0
                          % 2 no special treatment for p=0
basis.ExpendFlag=1;  % 1 = utility depends on f(y-p)
                     % 0 = utility depends on -f(p)
basis.EquivalenceScaleFlag=0;  % 1 = utility = f( (y-p)/N)
                               % 0 = otherwise (i.e. N=1)
                     
basis.scale          = 1;  % scaling factor for basis functions
basis.PriceScaleFlag = 0;  % 0 no scaling
                           % 1 rescale expenditure to have unit variance
                           % 2 divide by 100
if basis.flag==1
  % f(p) = cubic
  %        constraint:   basis.n=3
  basis.n      = 3;
elseif basis.flag==2
  % f(p) = series of B-splines
  %        constraints:
  %        nKnots >= 2
  %        k      >= 1
  %        n      = nKnots+k-1
  %        KnotFlag = 2 or 4
  basis.nKnots   = 8;
  basis.k        = 3;
  basis.n        = basis.nKnots+basis.k-1;
  basis.KnotFlag = 2;  % 4: knots at (0,2,4)';
                       % 2: knots at quantile of expenditure
elseif basis.flag==3
  % Chebyshev polynomials
  % Constraints
  %    nodes = 1
  %    k     >=1
  %    n     = k+nodes-1
  basis.nodes  = 1;  % 1 for Lobatto nodes
  basis.k      = 3;  % highest order polynomial
  basis.n      = basis.k+basis.nodes-1;
elseif basis.flag==4
  % linear utility
  %   Constraint:
  %   n = 1
  basis.n      = 1;
elseif basis.flag==5
  % log-linear utility
  % Constraint: 
  %   n = 1
  basis.n      = 1;
elseif basis.flag==6
  % cubic in log price
  %  Constraint:   n = 3
  basis.n=3; 
end

% Column indexes in raw data corresponding to each variable
% RawColumn.XX     columns in data containing XX
RawColumn.HH           = 2; % HH id  
RawColumn.HHTimePeriod = 1; % time peroid
RawColumn.Choice       = 6; % Choice=1    Product was chosen by HH, 0 otherwise

RawColumn.Regressors = [(55:57)';    ... % promo, ads, ads^2
                        (13:16)';    ... % pack size * HHSize
                        (108:110)';  ... % zz1 * labour==(2:4)'
                        (122:2:128)'];   % zz1 * hhmember==(2:5)'

RawColumn.PriceScaled       = 97; % p / hheqsize
RawColumn.ExpenditureScaled = 95; % (y-p) / hheqsize 
RawColumn.Price       = 7;  % p
RawColumn.Expenditure = 8;  % (y-p)
%  RawColumn.Z  = list of non-price variables to interact with random 
%                 coefficients
RawColumn.Z           = 11; % butter (interacted with random coefficients)
RawColumn.Interaction = (116:119)'; %(55:57)';  % variables to be interacted with f(p)
RawColumn.Sats        = 54; % column containing volsats (g/kj/kcal)
RawColumn.Sats_X      = 19; % sats interacted with x variables
RawColumn.Month       = 5;
RawColumn.Region      = [];
RawColumn.FixedEffect = (22:53)';   % product dummies
RawColumn.Brand       = 3;      
RawColumn.ProductCode = 3;
RawColumn.SampleProbability=0;  % prob HH included in sample (0 if no weights)
RawColumn.SampleWeight= 4;      % grossing up weight
RawColumn.ZeroRow     = [];     % indicator for outside option
RawColumn.EquivalenceScale = 70;

% subset of variables in raw data are extracted and stored in matrix X
% all columns in X that depend on price
% nPrice = number of variables in model that depend on price
% EstColumn.Expenditure = column numbers for variables that interact with
%                         expenditure
nPrice                = basis.n*(1+length(RawColumn.Interaction));
EstColumn.Expenditure = (1:nPrice)';

% These can be left empty for all calculations
iFree1.FixedBeta = [];
iFree2.FixedBeta = [];
iFree1.beta0     = [];
iFree2.beta0     = [];
iFree2.CDiag0    = [];
iFree2.COffDiag0 = [];

% CovarianceFlag = 0 : off-diagonal elements of covariance matrix restricted to be zero
%                = 1 : unrestricted
CovarianceFlag         = 1;
LogNormalFlag          = 0;  % 1 = RC on price is log normal

% RCDim:    Dimension of vector of random coefficients
% DeltaZIndex: List of columns containing variable to be interacted with
%              random coefficients
%    e.g.
%      DeltaZIndex=(5:6)';    interacts columns 5-6 with random
%                             coefficients
FirstModel = 1;  % Model = 1 logit model
LastModel  = 2;  % Model = 2 random coefficients logit model
RCDim      = 2;  % Number of variables that interact with random coefficients
RCRank     = 2;  % dimension of random coefficients
                 % Constraint 1 <= RCRank <= RCDim

% columns in raw data corresponding to random coefficients
% Z = subset of regressors with random coefficients
% ZIndex.iF    = columns in Z corresponding to f(y-p)
% ZIndex.iP    = columns in Z that depend on p
% ZIndex.iNonP = columns in Z that do not depend on p
ZIndex.iF        = 1;
ZIndex.iP        = 1;
ZIndex.iNonP     = max(ZIndex.iP)+(1:size(RawColumn.Z,1))';
ZIndex.RawColumn = RawColumn.Z;  
LogNormalIndex   = ZIndex.iP;

% Choose integration rule
% SparseGridFlag = 0 : Gauss-Hermite quadrature
%                = 1 : Sparse grids
%                = 2 : monte carlo integration
%                = 3 : monte carlo integration with antithetic variates
SparseGridFlag = 0;
MCDrawFlag     = 2;  % 1:  same draws for all HH
                     % 2:  different draws for each HH

if SparseGridFlag==0
  if RCRank<=5
    nQuad=5*ones(RCRank,1);
  elseif RCRank>5 && RCRank<9
    disp(['RCRank = ',RCRank]);
    error('Code does not yet work for RCRank>5 && RCRank<9')
  elseif RCRank==9
    nQuad=[5;5;3*ones(7,1)];
  else
    disp(['RCRank = ',RCRank]);
    error('Code does not yet work for RCRank>9')
  end
  nQuadAll = prod(nQuad);
elseif SparseGridFlag==1
  SparseLevel = 2  ;
  nQuadAll = sparse_grid_herm_size ( RCRank, SparseLevel );
elseif SparseGridFlag==2
  nQuadAll = 100;
elseif SparseGridFlag==3
  nQuadAll = 100;
end

% IncludeOutsideOption = 1 Outside option is included.
IncludeOutsideOption = 1;
OutsideOptionCode    = 1;
OutsideOptionColumn  = 1;

% Set options for optimization
OptimizerFlag = [3;3];  % 1 = fmincon
                        % 2 = knitro
                        % 3 = NAG
AlgFlag       = [1;1];  % 1 = interior-point
                        % 2 = SQP
                        % 3 = Active set
                        % 4 = trust-region-reflective
TestLikeFlag  = [0;0];  % 1 = test gradient of likelihood

options.MaxIter1 = 5000;
options.MaxIter2 = 5000;
options.DerivativeCheck = [];