function [l,dl]=like2(xFree,iFree,beta,CDiag,COffDiag,DeltaX,DeltaZ, ...
                      noptions,nobs,RCDim,RCRank,  ...
                      LogNormalFlag,LogNormalIndex, ...
                      Nodes,Weights,nQuad,  ...
                      OutDir)
%
% like2:  NORMAL MIXED LOGIT
%          (same integration nodes for each household)
%
%
% l	             = (1 x 1) value of likelihood
% dl             = (nFree x 1) gradient of likelihood
% xFree          = (nFree x 1) vector of free parameters
% iFree.beta{1}  = (nFreeBeta x 1) indexes in beta corresponding to free parameters
% iFree.beta{2}  = (nFreeBeta x 1) indexes in xFree corresponding to beta parameters
% iFree.CDiag{1} = (nFreeCDiag x 1) indexes in CDiag corresponding to free parameters
% iFree.CDiag{2} = (nFreeCDiag x 1) indexes in xFree corresponding to CDiag parameters
% iFree.COffDiag{1} = (nFreeCOffDiag x 1) indexes in COffDiag corresponding to free parameters
% iFree.COffDiag{2} = (nFreeCOffDiag x 1) indexes in xFree corresponding to COffDiag parameters
% beta           =  (nBeta x 1) all parameters in beta
% CDiag          =  (RCDim x 1) diagonal of C where S = C*C'
% COffDiag       =  (RCDIm*(RCDim-1)/2 x 1) off-diagonal of C
% DeltaX         =  (noptions*nobs x k) data matrix (sparse)
% DeltaZ         =  (nHH*noptions x RCDim) Data to interact with random coefficients
% nobs          =  (1 x 1) number of observations
% noptions      =  (1 x 1) number of options
% RCDim         =  (1 x 1) dimension of vector of random coefficients
% Nodes         =  (nQuad x RCDim) nodes for numerical integration
% Weights       =  (nQuad x 1) weights for numerical integration
% nQuad         =  (1 x 1) number of integration nodes
%
%
% All X variables are differenced
% DeltaX (noptions*nobs x k) product characteristics interacted with 
%                                  hh characteristics
%
% Revision history
%
% 2017MAR18  cleaned up version for QE.

% Save xFree after every iteration.
SaveIterationsFlag=0;
if SaveIterationsFlag==1
    SaveCurrentIteration(xFree,2,OutDir,1);
end

TuningFlag=2;
if TuningFlag==1
  small=1e-321;        % prevent taking log of zero
  big  = log(10^20);   % upper bound on dv1 to prevent isinf(dv1)
elseif TuningFlag==2
  small = 1e-321;
  big   = 600;
end

nFree=length(xFree);

% use iFree to map xFree to (beta,C)
%   xFree     free parameters
%   (beta,C)  structural parameters
[beta,C]=MapFreeToStructural(xFree,iFree,beta,CDiag,COffDiag,RCDim);
C = C(:,1:RCRank);

% Random coefficients
% size(Nodes) = (nQuadAll x RCRank)
% size(e) = (nQuadAll x RCDim)
e=Nodes*C.';
if LogNormalFlag==1
  e(:,LogNormalIndex) = -exp(e(:,LogNormalIndex));
end
dv1=repmat(DeltaX*beta,[1 nQuad]);

RCMethod=1;   % 1   vectorized version  (requires a lot of memory)
              % 2   loop version
              
if RCMethod==1              
  for i1=1:RCDim
    % size(e1) = (noptions*nobs x nQuad)
  
    e1=repmat(e(:,i1),[1 noptions*nobs]).';
    % size(dv1) = (noptions*nobs x nQuadAll)
    % size(DeltaZ) = (noptions*nobs x RCDim)
    dv1=dv1+e1.*repmat(DeltaZ(:,i1),[1 nQuad]);
  end
  clear e1 
elseif RCMethod==2
  for i1=1:RCDim
  for i2=1:nQuad
    dv1(:,i2)=dv1(:,i2)+e(i2,i1)*DeltaZ(:,i1);
  end 
  end

  % size(dv1) = (noptions*nobs x nQuadAll)
  % size(DeltaZ.zi) = (noptions*nobs x nQuadAll)
  %eval(['dv1=dv1+e1.*DeltaZ.z',int2str(i),';'])
end
dv1=min(dv1,big);
dv1=exp(dv1);

if nargout==1
  % exponentiate and sum across options to obtain:
  % size(dv1) = (nobs x nQuad)
  dv1=squeeze(sum(reshape(dv1,noptions,nobs,nQuad),1));
  
  if any(dv1==0)
    disp('dv1 = 0')
    keyboard
  end
  
  % Probability of choice: conditional on epsilon
  % size(dv1)  = (nobs x nQuad)
  % size(prob) = (nobs x 1)
  prob=(1./dv1)*Weights;                  % probability of choice
  
  l=sum(log(small+prob));
  l=-l;

elseif nargout==2
    
  % size(dv1) = (noptions*nobs x nQuad)  
  % exponentiate and sum across options to obtain t1:
  % size(t1) = (nobs x nQuad)
  % This is matrix of choice probabilities, one for each pair of
  % (household, shock)
  t1=squeeze(sum(reshape(dv1,noptions,nobs,nQuad),1));
  
  % size(prob) = (nobs x nQuad)
  prob=1./t1;
    
  % size(dv1) = (noptions*nobs x nquad)
  numerator=zeros(nobs,nFree);
  nFreeBeta=length(iFree.beta{1});

  iGrad.c = cell(RCDim,RCRank);
  
  for i1=1:RCDim
    if i1<=RCRank
      % xFree2(iGrad.c{i1,i1}) = C(i1,i1)
      if any(iFree.CDiag{1}==i1)
        iTemp = iFree.CDiag{2};
        iGrad.c{i1,i1} = iTemp(iFree.CDiag{1}==i1);
      end
    end
    
    for i2=1:min(i1-1,RCRank)
      % xFree2(iGrad.c{i1,i2}) = C(i1,i2)
      index = (i1-1)*(i1-2)/2+i2;
      if any(iFree.COffDiag{1}==index)
        iTemp = iFree.COffDiag{2};
        iGrad.c{i1,i2} = iTemp(iFree.COffDiag{1}==index);
      end
    end
  end

  for i1=1:nQuad
    % Sum over options in choice set of dv1.*DeltaX, given current random coefficient shock  
    % size(dv1) = (noptions*nobs x nQuad)
    % size(t2) = (noptions*nobs x nFreeBeta)
    
    % The next two lines use up 80% of execution time
    t2=repmat(dv1(:,i1),[1 nFreeBeta]);
    t2=t2.*DeltaX(:,iFree.beta{1});
    t2=reshape(t2,noptions,nobs*nFreeBeta);
    t3=sum(t2,1);
    t3=reshape(t3,nobs,nFreeBeta);
    t4=repmat(prob(:,i1),[1 nFreeBeta]);
    numerator(:,1:nFreeBeta)=numerator(:,1:nFreeBeta)- ...
                           Weights(i1)*             ...
                           (t3.*t4.*t4);
    
    for j1=1:RCRank
      
      if j1<=RCDim
        % Gradient w.r.t. C(j1,j1)
        if any(iFree.CDiag{1}==j1)
          t2 = Nodes(i1,j1)*dv1(:,i1).*DeltaZ(:,j1);
          if LogNormalFlag==1 && any(j1==LogNormalIndex)
            t2 = t2.*e(i1,j1);    
          end
          t2=reshape(t2,noptions,nobs);
          t2=sum(t2,1).';
          numerator(:,iGrad.c{j1,j1})=numerator(:,iGrad.c{j1,j1})- ...
                                      Weights(i1)*                 ...
                                      (t2.*prob(:,i1).*prob(:,i1));
        end
      end
      
      for j2=1:j1-1
        % Gradient w.r.t. c(j1,j2)
        index = (j1-1)*(j1-2)/2+j2;
        if any(iFree.COffDiag{1}==index)
          t2 = Nodes(i1,j2)*dv1(:,i1).*DeltaZ(:,j1);
          if LogNormalFlag==1 && any(j1==LogNormalIndex)
            t2 = t2.*e(i1,j1);      
          end
          t2=reshape(t2,noptions,nobs);
          t2=sum(t2,1)';
          numerator(:,iGrad.c{j1,j2})=numerator(:,iGrad.c{j1,j2})- ...
                                      Weights(i1)*              ...
                                      (t2.*prob(:,i1).*prob(:,i1));
        end % if any
      end   % for j2=1:j1-1
    end     % for j1=1:RCRank
  end       % for i1=1:nQuad
        
  % Probability of choice: conditional on epsilon
  % size(t1)  = (nobs x nQuad)
  % size(prob) = (nobs x 1)
  prob=small+prob*Weights;            % probability of choice
  
  l=-sum(log(prob));
  
  dl=-numerator.'*(1./prob);
  
end
if isnan(l)
    disp('l is nan');
    keyboard
end
