function [l,dl,Hess]=like3(xFree,iFree,beta,CDiag,COffDiag,DeltaX,DeltaZ, ...
                           noptions,nHH,nT,RCDim,RCRank,  ...
                           LogNormalFlag,LogNormalIndex, ...
                           Nodes,Weights,nQuad,  ...
                           OutDir)
%
% NORMAL MIXED LOGIT
%  (multiple observations per household)
%  (same integration nodes for all households)
%
%
%
% l	             = (1 x 1) value of likelihood
% dl             = (nFree x 1) gradient of likelihood
% xFree          = (nFree x 1) vector of free parameters
% iFree.beta{1}  = (nFreeBeta x 1) indexes in beta corresponding to free parameters
% iFree.beta{2}  = (nFreeBeta x 1) indexes in xFree corresponding to beta parameters
% iFree.CDiag{1} = (nFreeCDiag x 1) indexes in CDiag corresponding to free parameters
% iFree.CDiag{2} = (nFreeCDiag x 1) indexes in xFree corresponding to CDiag parameters
% iFree.COffDiag{1} = (nFreeCOffDiag x 1) indexes in COffDiag corresponding to free parameters
% iFree.COffDiag{2} = (nFreeCOffDiag x 1) indexes in xFree corresponding to COffDiag parameters
% beta           =  (nBeta x 1) all parameters in beta
% CDiag          =  (RCDim x 1) diagonal of C where S = C*C'
% COffDiag       =  (RCDIm*(RCDim-1)/2 x 1) off-diagonal of C
% DeltaX         =  (noptions*nHH x k) data matrix (sparse)
% DeltaZ         =  (noptions*nHH*nT x RCDim) Data to interact with random coefficients
% nHH          =  (1 x 1) number of households
% nT            =  (1 x 1) number of observations per household
% noptions      =  (1 x 1) number of options
% RCDim         =  (1 x 1) dimension of vector of random coefficients
% Nodes         =  (nQuad x RCDim) nodes for numerical integration
% Weights       =  (nQuad x 1) weights for numerical integration
% nQuad         =  (1 x 1) number of integration nodes
%
%
% All X variables are differenced
% DeltaX (noptions*nHH x k) product characteristics interacted with 
%                                  hh characteristics
%
% Revision history
%
% 2017MAR18  cleaned version for publication.

% Save xFree after every 100 iterations
SaveIterationsFlag=0;
if SaveIterationsFlag==1
  SaveCurrentIteration(xFree,2,OutDir,1);
end

TuningFlag=2;
if TuningFlag==1
  small=1e-321;        % prevent taking log of zero
elseif TuningFlag==2
  small = 1e-321;
end

nFree=length(xFree);

% use iFree to map xFree to (beta,C)
%   xFree     free parameters
%   (beta,C)  structural parameters
[beta,C]=MapFreeToStructural(xFree,iFree,beta,CDiag,COffDiag,RCDim);
C = C(:,1:RCRank);

% Random coefficients
% size(Nodes) = (nQuadAll x RCRank)
% size(e) = (nQuadAll x RCDim)
e=Nodes*C.';
if LogNormalFlag==1
  e(:,LogNormalIndex) = -exp(e(:,LogNormalIndex));
end
dv1=repmat(DeltaX*beta,[1 nQuad]);

RCMethod=1;
if RCMethod==1
  for i1=1:nQuad
    % size(e) = (nQuad x RCDim)
    dv1(:,i1) = dv1(:,i1) + DeltaZ*e(i1,:).'; 
  end
else
  for i1=1:RCDim
    % size(e1) = (noptions*nHH*nT x nQuad)
    e1=repmat(e(:,i1),[1 noptions*nHH*nT]).';
    % size(dv1) = (noptions*nHH*nT x nQuadAll)
    % size(DeltaZ) = (noptions*nHH*nT x RCDim)
    dv1=dv1+e1.*repmat(DeltaZ(:,i1),[1 nQuad]);
  end
end
clear e1 

% size(dv1)   = (noptions*nHH*T x nQuad)
% size(MaxDV) = (1 x nT*nHH x  nQuad)
% size(MaxDV) = (nHH x 1)
% MaxDV = max across j of dv, for each (t,n,epsilon)
[MaxDV,iMax] = max(reshape(dv1,[noptions nT*nHH nQuad]),[],1);

dv1 = dv1-reshape(repmat(MaxDV,[noptions 1 1]),[noptions*nT*nHH nQuad]);
dv1=exp(dv1);

MaxDV = reshape(exp(-MaxDV),[nT nHH nQuad]);
iMax  = reshape(iMax,[nT*nHH nQuad]);

if nargout==1
  % exponentiate and sum across options to obtain:
  % size(dv1) = (nT x nHH x nQuad)
  dv1=squeeze(sum(reshape(dv1,noptions,nT,nHH,nQuad),1));
  
  if any(dv1==0)
    disp('dv1 = 0')
    keyboard
  end
  
  % Probability of choice: conditional on epsilon
  % size(dv1)  = (nT x nHH x nQuad)
  % compute product of observations across time
  dv1 = squeeze(prod(MaxDV./dv1,1));
  % size(dv1) = (nHH x nQuad)
  % size(prob) = (nHH x 1)
  prob=dv1*Weights;                  % probability of choice
  
  l= sum(log(small+prob));
  l=-l;

elseif nargout>1
  % size(dv1) = (noptions*nT*nHH x nQuad)  
  % sum across options to obtain t1:
  % size(t1) = (nT x nHH x nQuad)
  % This is matrix of choice probabilities, one for each pair of
  % (household, time, shock)
  t1=squeeze(sum(reshape(dv1,noptions,nT,nHH,nQuad),1));
  
  % size(probT) = (nT x nHH x nQuad)
  probT = MaxDV./t1;
  
  % size(prob0) = (nHH x nQuad)
  % compute product across t
  prob0 = squeeze(prod(probT,1));
    
  % size(dv1) = (noptions*nT*nHH x nQuad)
  numerator=zeros(nHH,nFree);
  nFreeBeta=length(iFree.beta{1});

  iGrad.c = cell(RCRank,RCDim);
  
  for i1=1:RCDim
    if i1<=RCRank
      % xFree2(iGrad.c{i1,i1}) = C(i1,i1)
      if any(iFree.CDiag{1}==i1)
        iTemp = iFree.CDiag{2};
        iGrad.c{i1,i1} = iTemp(iFree.CDiag{1}==i1);
      end
    end
    
    for i2=1:i1-1
      % xFree2(iGrad.c{i1,i2}) = C(i1,i2)
      index = (i1-1)*(i1-2)/2+i2;
      if any(iFree.COffDiag{1}==index)
        iTemp = iFree.COffDiag{2};
        iGrad.c{i1,i2} = iTemp(iFree.COffDiag{1}==index);
      end
    end
  end

  parfor i1=1:nQuad
    TempNumerator = like3grad(i1,dv1(:,i1),t1(:,:,i1),iMax(:,i1), ...
                              prob0(:,i1),Nodes(i1,:),e(i1,:), ...
                              DeltaX,DeltaZ, ...
                              iFree,iGrad,RCDim,RCRank,noptions,nT,nHH, ...
                              nFreeBeta,nFree,LogNormalFlag,LogNormalIndex);
    numerator = numerator + Weights(i1)*TempNumerator;     
  end       % for i1=1:nQuad
        
  % Probability of choice: conditional on epsilon
  % size(t1)  = (nHH x nQuad)
  % size(prob) = (nHH x 1)
  prob0=small+prob0*Weights;            % probability of choice
  
  l=-sum(log(prob0));
  dl = - sum(numerator./repmat(prob0,[1 nFree])).';
  
  if nargout==3
    % size(temp) = (k x N)
    temp = (numerator./repmat(prob0,[1 nFree])).';
    temp1 = reshape(temp,[nFree 1 nHH]);
    temp1 = repmat(temp1,[1 nFree 1]);
    temp2 = reshape(temp,[1 nFree nHH]);
    temp2 = repmat(temp2,[nFree 1 1]);
    Hess = sum(temp1.*temp2,3);
  end
end
if isnan(l)
    disp('l is nan');
    keyboard
end
