
% print out elapsed computation time
TimeFactor = 3600;   % 60 for minutes
                     % 3600 for hours

% Start up parallel pool in matlab distributed computing engine                     
v=version('-release');             
if strncmp(v,'2011',4) || strncmp(v,'2012',4)                         
  pp =matlabpool('size');
  if pp>0
    matlabpool('close');
  end
  matlabpool('open');
else
  if ~isempty(gcp('nocreate'))
    delete(gcp('nocreate'));
  end
  parpool;
end
        
Time.Start=clock;

if ~exist('ParameterFile','var')
  display('User needs to define ParameterFile.');
  error('For example: ParameterFile = ''inputs/V0''.');
end

%% Main file to estimate random coefficients logit model using Kantar data
%
% Revision history
%
% 2017MAR18  Final version for publication

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
format short e

% Add directories to path
setpath;

if exist('ParameterFile','var')==0
  display('You must first define a string containing the name of a valid parameter file.');
  display('e.g.:  ParameterFile = ''inputs/V0_A''');
  display('In this example, the name of the parameter file is inputs/V0_A.m');
  return;
else
  if exist([ParameterFile,'.m'],'file')==0
    display([ParameterFile,'.m cannot be found.']);
    display('Please specify valid parameter file location.');
    return;
  end
end

% Read parameters from parameter file.
eval(ParameterFile);

timings=zeros(2,1);

%% Load and prepare data
%   If DataFile omitted, assume default data file.
if raw==1
  % Load and clean data
  DataHeaderLine=1;
  aa=clock;
  display(['Begin loading of data from: ',RawDataFile]);
  if aa(5)<10
    display(['Program start time: ',int2str(aa(4)),':',int2str(0), ...
                                    int2str(aa(5))]);
  else
    display(['Program start time: ',int2str(aa(4)),':',int2str(aa(5))]);
  end
  [data,SumStats,RawVarLabels]=    ...
   CleanData(RawDataFile,DataHeaderLine,SaveDataFile,SaveDataFlag, ...
             SummaryFile, ...
             nHH_Raw,noptions,nvars,nT,nHH,       ...
             RawColumn.HH,RawColumn.Choice,RawColumn.ProductCode);
else
  % (initialise values of (nHH,data.data,noptions)
  nHH=0;
  data.data=0;
  noptions=0;
  eval(['load ',SaveDataFile]);
  RawVarLabels=VarLabels;
end

% Size of data
if isstruct(data)
  data=data.data;
end

data(:,RawColumn.Sats)   = data(:,RawColumn.Sats)/100;
data(:,RawColumn.Sats_X) = data(:,RawColumn.Sats_X)/100;
if basis.PriceScaleFlag==1
  % rescale to have variance 1
  index1                 = data(:,RawColumn.Choice)==1;  
  basis.ExpenditureScale = sqrt(var(data(index1,RawColumn.Expenditure)));
  data(:,RawColumn.Expenditure) = data(:,RawColumn.Expenditure)/basis.ExpenditureScale;
  data(:,RawColumn.Price)       = data(:,RawColumn.Price)/basis.ExpenditureScale;
elseif basis.PriceScaleFlag==2
  basis.ExpenditureScale = 100;
  % rescale to have variance 1
  data(:,RawColumn.Expenditure) = data(:,RawColumn.Expenditure)/basis.ExpenditureScale;
  data(:,RawColumn.Price)       = data(:,RawColumn.Price)/basis.ExpenditureScale;
end

% Compute DeltaX = x(i,j) - x(i,noptions)
% the difference between the x's for option j and
% the x's for the option chosen

% Compute product fixed effect dummy matrix.
% and X_FE = matrix of regressors to compute FE regression after
% estimating logit model
BrandDummies     = data(:,RawColumn.FixedEffect);
HHProductCodeRaw = data(:,RawColumn.ProductCode);  % product codes in HH level dataset
HHCode           = data(:,RawColumn.HH);

% Create DeltaX and update basis (define knots etc.)
% Likelihood function depends on difference across products of regressors
% DeltaX = x(:,j) - x(:,J)
[DeltaX,DeltaZ,NotIdentifiedBrands,basis] = ...
    CreateDeltaX(data,BrandDummies,RawColumn,ZIndex,basis, ...
                 nHH*nT,noptions,RCDim);
nRegressors          = length(RawColumn.Regressors);                                              
[n,nbeta]            = size(DeltaX);
DeltaXLabels         = cell(nbeta,1);
DeltaX(:,1:basis.nf) = DeltaX(:,1:basis.nf)/basis.scale;
DeltaZ(:,ZIndex.iP)  = DeltaZ(:,ZIndex.iP)/basis.scale;

% labels for interactions between f(p) and X
for i1=0:length(RawColumn.Interaction)
   for i2=1:basis.nf
     index = i1*basis.nf + i2;
     if i1==0
       DeltaXLabels{index} = ['p',int2str(i2)];    
     else
       DeltaXLabels{index} = ...
           [RawVarLabels{RawColumn.Interaction(i1)},'_p',int2str(i2)];
     end
   end
end
DeltaXLabels(index+(1:nRegressors)')=RawVarLabels(RawColumn.Regressors);
DeltaXLabels(index+nRegressors+(1:length(RawColumn.FixedEffect))') = ...
    RawVarLabels(RawColumn.FixedEffect);
DeltaZLabels = cell(length(ZIndex.iP)+length(ZIndex.iNonP),1);
for i1=1:length(ZIndex.iP)
  DeltaZLabels{i1} = ['p',int2str(ZIndex.iF(i1))];
end
DeltaZLabels(length(ZIndex.iP)+(1:length(ZIndex.iNonP))') = ...
    RawVarLabels(ZIndex.RawColumn);

% Save data file for later use and clear from memory
if SaveDataFlag==1
  save(SaveDataFile,'data','nHH','nT','HHCode','HHProductCodeRaw','basis', ...
       'BrandDummies','-v7.3')
end
clear data HHCode HHProductCodeRaw BrandDummies;

%% First estimate model 1: simple logit
% Then estimate RC logit: using model 1 estimates as guess.
Time.LoadData=clock;
if Time.LoadData(5)<10
  disp(['Current time : ',num2str(Time.LoadData(4)),':',int2str(0), ...
                          num2str(Time.LoadData(5))]);
else
  disp(['Current time : ',num2str(Time.LoadData(4)),':', ...
                          num2str(Time.LoadData(5))]);
end
disp(['Finished data cleaning. Elapsed time: ', ...
      num2str(etime(Time.LoadData,Time.Start)/60),' minutes']);

% number of regressors and initial values for beta
nbeta1=nbeta;
nbeta2=nbeta;
beta1 = zeros(nbeta1,1);

% Set optimization options.
options.model1=optimset('Display','iter','MaxFunEvals',50000, ...
                        'DerivativeCheck','off',              ...
                        'GradObj','on',                       ...
                        'LargeScale','off', ...
                        'RelLineSrchBnd',2);
options.model1=optimset(options.model1,'MaxIter',options.MaxIter1);
options.model2=optimset(options.model1,'MaxIter',options.MaxIter2);
options.model1=SetOptions(options.model1,AlgFlag(1));
options.model2=SetOptions(options.model2,AlgFlag(2));


for model=FirstModel:LastModel
% model==1 : Estimate logit model 1.
% model==2 : Estimate random coefficients logit model 2.

save(EstimationDataFile,'DeltaX','-v7.3');

%% Estimate logit model 1
if model==1

   % Vector of parameters to estimate
   % xFree1
   % 1:nbeta1              beta_observed
   %
   % Vector of variables set at fixed values
   % xFix
   % none

   % Indexes of free parameters
   iFree1.beta0 = (1:nbeta1)';
   if ~isempty(iFree1.FixedBeta)
     index = true(nbeta1,1);
     for i1=1:length(iFree1.FixedBeta)
       index = index & ((1:nbeta1)'~=iFree1.FixedBeta(i1));      
     end
     iFree1.beta0 = iFree1.beta0(index);
   end
      
   if ~isempty(NotIdentifiedBrands);
     iFree1.beta=RemoveIndex(iFree1.beta0,NotIdentifiedBrands);
   else
     iFree1.beta=iFree1.beta0;
   end
   nFree1=length(iFree1.beta);

   % initial guess for solution
   xFree1=beta1(iFree1.beta);

   % Lower and upper bounds on parameters
   [lower1,upper1]=SetBounds1(xFree1,iFree1.beta,DeltaX);

   LikeFunc1 = @(x)like1(x,iFree1.beta,nFree1,beta1,DeltaX,noptions,nHH*nT,OutDir);

   if TestLikeFlag(1)==1
     GradErr1 = TestLike(LikeFunc1,xFree1,1);
     keyboard;
   end
   
   % Save intial value of xFree1 and start counting iterations
   %    xFree is saved every 100 iterations.
   SaveCurrentIteration(xFree1,model,OutDir,0);
   
   % define constraints on optimization problem
   A = [];
   B = [];
   Aeq = [];
   Beq = [];
   confun = [];

   % Maximise likelihood function (minimise negative of likelihood)
   if OptimizerFlag(1)==1
     % Use fmincon
     tic
     [xoptimal1,~,exitflag1,output,lambda]=...
        fmincon(LikeFunc1,xFree1,A,B,Aeq,Beq,lower1,upper1,confun,options.model1);
     timings(1)=timings(1)+toc/TimeFactor;
     [LikeValue1,grad1,Hess1]=LikeFunc1(xoptimal1);
   elseif OptimizerFlag(1)==2
      tic
      % use Knitro
      [xoptimal1,~,exitflag1,output,lambda]=...
         fmin_wrapper(LikeFunc1,xFree1,A,B,Aeq,Beq,lower1,upper1,confun, ...
                      options.model1,OptimizerFlag(1));

      timings(1)=timings(1)+toc/TimeFactor;
      [LikeValue1,grad1,Hess1]=LikeFunc1(xoptimal1);
   elseif OptimizerFlag(1)==3
     tic
     CL     = [];
     CU     = [];
     % Use NAG
     [xoptimal1,~,exitflag1,output,lambda] = ...
        CallNAG(LikeFunc1,xFree1,A,B,Aeq,Beq, ...
                lower1,upper1,confun,CL,CU,options.model1);
     timings(1)=timings(1)+toc/TimeFactor;
     [LikeValue1,grad1,Hess1]=LikeFunc1(xoptimal1);
   end

   % Save results in beta1 and StandardErrors1
   [xFree1,beta1,StandardErrors1,BIC1] = ...
  PrintEstimationResults_Model1(xoptimal1,LikeValue1,grad1,Hess1,timings(1),TimeFactor, ...
                                iFree1,beta1,nFree1,nHH*nT,OutFile1,DeltaXLabels);

%% Estimate random coefficients model 2
elseif model==2

  nbeta2 = size(DeltaX,2);
  
  % Set up integration rule
  if SparseGridFlag==0
    for i1=1:RCRank
      [x,w]=gauher(nQuad(i1));
      if i1==1
        Nodes=x;
        Weights=w;
      else
        nQuad1=prod(nQuad(1:i1-1));
        Nodes=[kron(Nodes,ones(nQuad(i1),1)) kron(ones(nQuad1,1),x)];
        Weights=kron(Weights,ones(nQuad(i1),1)).*...
                kron(ones(nQuad1,1),w);
      end
    end
    Nodes=sqrt(2)*Nodes;
    Weights=Weights*pi^(-RCRank/2);
    nQuadAll=prod(nQuad);
  elseif SparseGridFlag==1
    [Weights,Nodes]=sparse_grid_herm(RCRank,SparseLevel,nQuadAll);
    Nodes = sqrt(2)*Nodes';
    Weights = (pi^(-RCRank/2))*Weights';
  elseif SparseGridFlag==2
    rng('default');
    Nodes = randn(nQuadAll,RCRank);
    Weights = ones(nQuadAll,1)/nQuadAll;
  elseif SparseGridFlag==3
    % pseudo-Monte Carlo with antithetic variates
    rng('default');
    if (round(nQuadAll/2)*2~=nQuadAll)
      nQuadAll=nQuadAll+1;
    end
    
    if MCDrawFlag==1
      % Same draws for each HH
      temp1 = randn(nQuadAll/2,RCRank);
      Nodes = [temp1;-temp1];
      Weights = ones(nQuadAll,1)/nQuadAll;
    elseif MCDrawFlag==2
      % Different draws for each HH
      Nodes = zeros(nHH*nQuadAll,RCRank);
      for i1=1:RCRank
        % [A1 A2;
        %  B1 B2]
        % [A1 A2 -A1 -A2;
        %  B1 B2 -B1 -B2]
        % A1
        % B1
        % A2
        % B2
        temp1 = randn(nHH,nQuadAll/2);
        Nodes(:,i1) = reshape([temp1, -temp1],[nHH*nQuadAll,1]);
      end
      Weights = ones(nQuadAll,1)/nQuadAll;
    end
  end

  save(EstimationDataFile,'DeltaX','DeltaZ', ...
                          'DeltaXLabels',    ...
                          'DeltaZLabels','nHH','nT','-v7.3');

  %  Define mapping from xFree2 to (beta2,C)
  %    xFree2    vector of free parameters
  %    (beta2,C) structural parameters
  %
  %  beta2(iFree2.beta{1}) = xFree2(iFree2.beta{2})
  %  C(    iFree2.CDiag{1}) = xFree2(iFree2.CDiag{2})
  %  C(
  %  

  % Select free parameters
  if isempty(iFree2.beta0)
    iFree2.beta0= (1:nbeta2)';
    if ~isempty(iFree2.FixedBeta)
      index = true(nbeta2,1);
      for i1=1:length(iFree2.FixedBeta)
        index = index & (iFree2.beta0~=iFree2.FixedBeta(i1));
      end
      iFree2.beta0 = iFree2.beta0(index);
    end
  end
  
  % Free parameters after removing NotIdentifiedBrands
  iFree2.beta = cell(2,1);
  iFree2.beta{1}=RemoveIndex(iFree2.beta0,NotIdentifiedBrands);
  iFree2.nBeta=length(iFree2.beta{1});
  iFree2.beta{2}=(1:iFree2.nBeta)';

  CFlag=1;
  if CFlag==1
    CDiag = zeros(RCDim,1);
    COffDiag = zeros(RCDim*(RCDim-1)/2,1);
  end
  if LogNormalFlag==1
    CDiag(LogNormalIndex) = 0.1;
  end


  %  mapping from xFree2 to C
  %
  %   Covariance matrix is  sig=C*C'
  %   C is initially stored as vector of size (RCDim*RCDim x 1).
  %     the upper triangle of C is constrained to equal 0
  %   e.g. RCDim = 3
  %     C(1,1) C(1,2) C(1,3)
  %     C(2,1) C(2,2) C(2,3)
  %     C(3,1) C(3,2) C(3,3)
  % Then C is converted to a vector
  %   C(iFree2.C{1}) = xFree2(iFree2.C{2})
  iFree2.CDiag    = cell(2,1);
  iFree2.COffDiag = cell(2,1);
  if isempty(iFree2.CDiag0)
    iFree2.CDiag{1} = (1:RCRank)';
  else
    iFree2.CDiag{1} = iFree2.CDiag0;
  end

  if CovarianceFlag==0
    CDiag(iFree2.CDiag{1}) = 1;
    iFree2.nCDiag = length(iFree2.CDiag{1});
    iFree2.CDiag{2} = iFree2.nBeta + (1:iFree2.nCDiag)';
    iFree2.COffDiag{1} = [];
    iFree2.nCOffDiag = length(iFree2.COffDiag{1});
    iFree2.COffDiag{2} = iFree2.nBeta+iFree2.nCDiag+(1:iFree2.nCOffDiag)';
  elseif CovarianceFlag==1
    CDiag(iFree2.CDiag{1})=1;
    iFree2.nCDiag = length(iFree2.CDiag{1});
    iFree2.CDiag{2} = iFree2.nBeta + (1:iFree2.nCDiag)';
    if isempty(iFree2.COffDiag0)
      iFree2.COffDiag{1}=CreateIFreeRank(RCDim,RCRank);
    else
      iFree2.COffDiag{1} = iFree2.COffDiag0;
    end
      iFree2.nCOffDiag = length(iFree2.COffDiag{1});
      iFree2.COffDiag{2} = iFree2.nBeta ...
                         + iFree2.nCDiag+(1:iFree2.nCOffDiag)';
  end

  iFree2.n=iFree2.nBeta+iFree2.nCDiag+iFree2.nCOffDiag;
  
  % Initial guess for beta2
  if FirstModel==1
    beta2=beta1;
  else
    if exist('beta0','var')
      beta2=beta0;
    else
      beta2=zeros(nbeta2,1)+0.1;
    end
  end

  % Complete list of parameters to estimate
  xFree2=[beta2(iFree2.beta{1});  ...
          CDiag(iFree2.CDiag{1}); ...
          COffDiag(iFree2.COffDiag{1})];

  % Upper and lower bounds on parameters
  lower2 = zeros(iFree2.n,1);
  upper2 = zeros(iFree2.n,1);
  if exist('lower1','var')
    if length(lower1)==iFree2.nBeta  
      lower2(iFree2.beta{2})=lower1;
      upper2(iFree2.beta{2})=upper1;
    else
      lower2 = xFree2-40;
      upper2 = xFree2+40;
    end
  else
    lower2 = xFree2 - 40;
    upper2 = xFree2 + 40;
  end
  lower2(iFree2.CDiag{2}) = -20;
  upper2(iFree2.CDiag{2}) = 50;
  lower2(iFree2.COffDiag{2}) = -40;
  upper2(iFree2.COffDiag{2}) = 40;

  % Choose likelihood function
  if nT==1
    % One observation per household
    LikeFunc2 = @(x)like2(x,iFree2,beta2,CDiag,COffDiag,DeltaX, ...
                           DeltaZ,noptions,nHH,RCDim,RCRank, ...
                           LogNormalFlag,LogNormalIndex,Nodes,Weights, ...
                           nQuadAll,OutDir);
  elseif nT>1
    % Multiple observations per household
    if MCDrawFlag==1 || SparseGridFlag<2
      % integration: use same Monte Carlo draws for each household
      %              for Gauss rules this is the right option
      LikeFunc2 = @(x)like3(x,iFree2,beta2,CDiag,COffDiag,DeltaX, ...
                            DeltaZ,noptions,nHH,nT,RCDim,RCRank, ...
                            LogNormalFlag,LogNormalIndex,Nodes,Weights, ...
                            nQuadAll,OutDir);
    elseif MCDrawFlag==2 && SparseGridFlag>1
      % integration: use different Monte Carlo draws for each HH
      %              for MC integration, this is correct option.
      LikeFunc2 = @(x)like4(x,iFree2,beta2,CDiag,COffDiag,DeltaX, ...
                            DeltaZ,noptions,nHH,nT,RCDim,RCRank, ...
                            LogNormalFlag,LogNormalIndex,Nodes,Weights, ...
                            nQuadAll,OutDir);
    end
  end

  % Test computation of gradient of likelihood if desired.
  if TestLikeFlag(2)==1
    GradErr2 = TestLike(LikeFunc2,xFree2);
    keyboard;
  end

  % Define constraints for optimization
  A = [];
  B = [];
  Aeq = [];
  Beq = [];
  confun = [];
  
  % Save intial value of xFree2 and start counting iterations
  %    xFree2 is saved every 100 iterations.
  SaveCurrentIteration(xFree2,model,OutDir,0);

  save(MatFile2,'-v7.3');
  if OptimizerFlag(2)==1
    % use fmincon
    tic
    [xoptimal2,~,exitflag2,output,lambda]= ...
       fmincon(LikeFunc2,xFree2,A,B,Aeq,Beq,lower2,upper2,confun,    ...
               options.model2);
    timings(2)=timings(2)+toc/TimeFactor;
    save(MatFile2,'-v7.3');
    [LikeValue2,grad2,Hess2]=LikeFunc2(xFree2);
    save(MatFile2,'-v7.3');
  elseif OptimizerFlag(2)==2
    tic
    % use Knitro
    [xoptimal2,~,exitflag2,output,lambda]=...
       fmin_wrapper(LikeFunc2,xFree2,A,B,Aeq,Beq,lower2,upper2,confun, ...
                    options.model2,OptimizerFlag(2));
    timings(2)=timings(2)+toc/TimeFactor;
    save(MatFile2,'-v7.3');
    [LikeValue2,grad2,Hess2]=LikeFunc2(xFree2);
    save(MatFile2,'-v7.3');
  elseif OptimizerFlag(2)==3
    % use NAG
    tic
    CL     = [];
    CU     = [];
    [xoptimal2,~,exitflag2,output,lambda] = ...
       CallNAG(LikeFunc2,xFree2,A,B,Aeq,Beq, ...
               lower2,upper2,confun,CL,CU,options.model2);
    timings(2)=timings(2)+toc/TimeFactor;
    save(MatFile2,'-v7.3');
    [LikeValue2,grad2,Hess2]=LikeFunc2(xoptimal2);
    save(MatFile2,'-v7.3');
  end
  [xFree2,beta2,C,CDiag,COffDiag,StandardErrors2,SIG,SIG_SE,BIC2] = ...
  PrintEstimationResults_Model2(xoptimal2,LikeValue2,grad2,Hess2,  ...
                                timings(2),TimeFactor,             ...
                                iFree2,beta2,CDiag,COffDiag,RCDim,RCRank, ...
                                iFree2.n,nHH*nT,OutFile2,     ...
                                DeltaXLabels,DeltaZLabels);
  save(MatFile2,'-v7.3');
end % if model==1
end % for model=1:2

Time.Estimation=clock;
disp(['Estimation complete. ',num2str(Time.Estimation(4)),':',num2str(Time.Estimation(5))])

% Save estimation results to disk
clear DeltaX DeltaZ;
if model==1
  xFree2 = xFree1;  
  beta2 = beta1;
  iFree2 = iFree1;
  CDiag = 0;
  COffDiag = 0;
  C = 0;
  LikeValue2 = LikeValue1;
  grad2 = grad1;
  Hess2 = Hess1;
  Nodes = 0;
  Weights = 0;
end
  save(EstimationResultsMatFile,'-v7.3', ...
       'xFree2',      ...
       'iFree2',      ...
       'beta2',       ...
       'CDiag',       ...
       'COffDiag',    ...
       'C',           ...
       'LikeValue2',  ...
       'grad2',       ...
       'Hess2',       ...
       'LogNormalFlag', ...
       'LogNormalIndex', ...
       'Nodes',       ...
       'Weights',     ...
       'nQuadAll');