function [MarketCost,HHCost]=ComputeCosts(S,GradS,HHWeights,Price, ...
          noptions,nHH,nFirms,Ownership,Market,nMarkets,OutsideOption)
% 
% Compute vector of costs in each market.
%
%
% function Cost=ComputeCosts(S,GradS,noptions,nHH,nFirms,AllFirmsProducts, ...
%                           Market,nMarkets)
%
% MarketCost    =  (noptions x nMarkets) Costs in each market
% HHCost        =  (noptions x nHH) marginal cost for each product implied by
%                                    profit maximization
%
% S	            =  (noptions x nHH) vector of market shares
% GradS         =  (noptions x noptions x nHH) Gradient of market share w.r.t. price
% HHWeights     =  (nHH x 1)  household weights for grossing up
% Price         =  (noptions x nMarkets)   prices for each (product,market)
%
% noptions      =  (1 x 1) number of options
% nHH           =  (1 x 1) number of observations
% nFirms        =  (1 x 1) number of firms
% OwnerShip     =  (nFirms x 1) (cell array) Element i is a list of indexes
%                               for products owned by firm i. That is,
%                               Ownership{i} lists the products owned by
%                               firm i.
% Market        =  (nHH x 1) index of market in which HH is included
% nMarkets      =  (1 x 1)  number of markets.
% OutsideOption =  (1 x 1)  FirmID of OutsideOption
%
%
% Revision history
% 2017MAR18  Cleaned version for publication.
MarketCost=zeros(noptions,nMarkets);
HHCost=zeros(noptions,nHH);

for f=1:nFirms
  % Indexes of Products owned by firm f
  
  if f~=OutsideOption
    CurrentProducts=Ownership{f};
    J=length(CurrentProducts);
  
    if J>0
    % One cost for each value of market
    for m=1:nMarkets
      i1= Market==m;
      nCurrent=sum(i1);
      if nCurrent>0
        TempS=sum(S(CurrentProducts,i1).*HHWeights(CurrentProducts,i1),2);
        TempWeight = permute(repmat(HHWeights(CurrentProducts,i1),[1 1 J]),[3 1 2]);
        TempGrad = sum(GradS(CurrentProducts,CurrentProducts,i1) ...
                       .*TempWeight,3);
      
        TempCost = Price(CurrentProducts,m) ...
               +TempGrad \ TempS; 
        MarketCost(CurrentProducts,m)=TempCost;     
        HHCost(CurrentProducts,i1)=repmat(TempCost,[1 nCurrent]);
      end
    end
    end
  end
end