function elas=Elasticity(X,Z,xFree,iFree,beta,CDiag,COffDiag, ... 
                         LogNormalFlag,LogNormalIndex,EstColumn,ZIndex, ...
                         noptions,nHH,RCDim,RCRank,HHProductCode, ...
                         Nodes,Weights,nQuad,SpendFlag)
%
% Compute elasticities w.r.t. price for
% mixed logit model 
%
%
% elas	        =  (noptions x noptions x nHH) matrix of elasticities
%                                               one matrix for each hh.
% elas(i,j,n)   = elas of j w.rt. price of i
% xFree         =  (nFree x 1) vector of free parameters
% iFree{1}      =  (n1 x 1) indexes in beta corresponding to free parameters
% iFree{2}      =  (n2 x 1) indexes in xFree corresponding to beta parameters
% iFree{3}      =  (n3 x 1) indexes in C corresponding to free parameters
% iFree{4}      =  (n4 x 1) indexes in xFree corresponding to C parameters
% beta          =  (nx    x 1) all parameters in x
% C
% X             =  (noptions*nHH x k) data matrix (sparse)
% Z             =  (noptions*nHH x RCDim) data matrix to interact with
%                                          random coefficients
% EstColumn.Expenditure   = (1 x 1) estimation data column of expenditure
% nHH           =  (1 x 1) number of observations
% noptions      =  (1 x 1) number of options
% RCDim         =  (1 x 1) dimension of vector of random coefficients
% Nodes         =  (nQuad x RCDim) nodes for numerical integration
% Weights       =  (nQuad x 1) weights for numerical integration
% nQuad         =  (1 x 1) number of integration nodes
%
%
% Revision history
% 2017MAR18  Cleaned version for publication.

% map xFree to (beta,C)
[beta,C]=MapFreeToStructural(xFree,iFree,beta,CDiag,COffDiag,RCDim);
C = C(:,1:RCRank);

% v1 = (J*N x nQuad) utility obtained from each product (including random coefficients)
% GradV1 = (J*N x nQuad)  gradient of v1 w.r.t. price
v1=repmat(X.X*beta,[1 nQuad]);
GradV1=full(repmat(X.GradX*beta(EstColumn.Expenditure),[1 nQuad]));

RCMethod=2; % 1: uses a lot of memory
            % 2: uses less memory but with an added loop

% Random coefficients
e=Nodes*C.';
if LogNormalFlag==1
  e(:,LogNormalIndex) = -exp(e(:,LogNormalIndex));
end

if RCMethod==1
  for i1=1:RCDim
    % size(e1) = (noptions*nHH x nQuad)
    if (size(e,1)==nQuad)
      % same draws for all households, must replicate e
      e1 = repmat(e(:,i1),[1 noptions*nHH]).';
    else
      % different draws for each HH
      nHHQuad = size(e,1)/nQuad;
      nT = nHH/nHHQuad;
      e1=kron(reshape(e(:,i1),[nHHQuad nQuad]),ones(nT*noptions,1));
    end

    % size(v1) = (noptions*nHH x nQuad)
    % size(Z.Z) = (noptions*nHH x RCDim)
    v1 = v1 + e1.*repmat(Z.Z(:,i1),[1 nQuad]);
    
    % if Z.GradZ(:,i1) depends on price, compute price impact
    if any(i1==ZIndex.iP)
      for i2=1:nQuad
        GradV1(:,i2) = GradV1(:,i2) + e1(:,i2).*Z.GradZ(:,i1);
      end
    end
  end   % for i1=1:RCDim
elseif RCMethod==2
  for i1=1:RCDim
  for i2=1:nQuad
    if (size(e,1)==nQuad)
      v1(:,i2) = v1(:,i2) + e(i2,i1)*Z.Z(:,i1);
      if any(i1==ZIndex.iP)
        GradV1(:,i2) = GradV1(:,i2) + e(i2,i1)*Z.GradZ(:,i1);
      end
    else
      % different draws for each HH
      nHHQuad = size(e,1)/nQuad;
      nT =nHH/nHHQuad;
      e1 = kron(e((i2-1)*nHHQuad+(1:nHHQuad)',i1),ones(nT*noptions,1));
      v1(:,i2) = v1(:,i2) + e1.*Z.Z(:,i1);
      if any(i1==ZIndex.iP)
        GradV1(:,i2) = GradV1(:,i2) + e1.*Z.GradZ(:,i1);
      end
    end
  end  % for i2=1:nQuad
  end  % for i1=1:RCDim
end     % if RCMethod==1

v1 = reshape(v1,[noptions nHH nQuad]);
MaxV1 = max(v1,[],1);
v1 = reshape(v1-repmat(MaxV1,[noptions 1 1]),[noptions*nHH nQuad]);
v1=exp(v1);

clear e1 zTemp;

% Sum across options, replicate and reshape to obtain denom with:
% size(denom) = (noptions*nHH x nQuad)
denom=sum(reshape(v1,noptions,nHH,nQuad),1);
denom=repmat(denom,[noptions 1 1]);
denom=reshape(denom,noptions*nHH,nQuad);
  
% Probability of choice average across epsilon
% size(prob0)  = (noptions*nHH x nQuad)
% size(prob)   = (noptions x nHH)
prob0=v1./denom;                           % probability conditional on epsilon
prob=reshape(prob0*Weights,noptions,nHH);  % HH specific probability averaged across epsilon

% Calculate elasticity of prob w.r.t. to prices
elas=zeros(noptions,noptions,nHH);
Price=reshape(X.Price,noptions,nHH);

for j1=1:noptions
  % elas = share * gradient / price
  % Pick out rows of prob0 and Gradv1 corresponding to CurrentProducts(j1)
  % Then compute weighted average across epsilon using Weights
  % size(temp) = (nHH x 1)
%  disp(j1)
  temp=(prob0(HHProductCode==j1,:) ...
        .*(1-prob0(HHProductCode==j1,:)) ...
        .*GradV1(HHProductCode==j1,:))*Weights;
  %elas(j1,j1,:)=reshape(Price(j1,:)'.*temp./prob(j1,:)',[1 1 nHH]);
  elas(j1,j1,:)=Price(j1,:)'.*temp./prob(j1,:)';
  for j2=1:noptions
    % Gradient of shares w.r.t. to cross-prices
    % Pick out rows of prob0 corresponding to products
    % j1 and j2 and derivative with respect to price of product j2
    if j2~=j1
      temp=-(prob0(HHProductCode==j1,:)   ...
             .*prob0(HHProductCode==j2,:) ...
             .*GradV1(HHProductCode==j2,:))*Weights;
%      elas(j2,j1,:)=reshape(Price(j2,:)'.*temp./prob(j1,:)',[1 1 nHH]);
      elas(j2,j1,:)=Price(j2,:)'.*temp./prob(j1,:)';
    end
  end
end

if SpendFlag==1
  elas = - elas;
end
