function [R,J]=Equilibrium2(Price,Cost,Tax,Sats,HHWeights,X,Z,iNonZero,basis,  ...
                            noptions,nHH,RCDim,RCRank,    ...
                            LogNormalFlag,LogNormalIndex,EstColumn,ZIndex,   ...
                            xFree2,iFree2,beta2,CDiag,COffDiag,nFirms, ... 
                            AllFirmsProducts,AllCompetitorProducts,    ...
                            NewHHProductCode,Nodes,Weights,nQuadAll,   ...
                            MaxN)
                       
% Compute vector of residuals of equilibrium conditions.
% Also compute Jacobian of the vector of equation
% For an equilibrium Price, residuals will all be zero.
%
% R         = (noptions x 1)  Vector of residuals for equilibrium equations
% J         = (noptions x noptions)  J(i,j) = derivative of R(i) w.r.t. P(j)
% Price     = (noptions x 1)  Vector of prices to evaluate residual
% Cost      = (noptions x 1)  Vector of marginal costs
% Tax.Type  = (1 x 1)         1 for excise, 2 for ad valorem
% Tax.tax   = (1 x 1)         excise rate: Tax per unit of saturated fats
% Tax.tau   = (1 x 1)         ad valorem rate: percent tax per unit fats   
% Sats      = (noptions x 1)  Volume of saturated fats in each product
% HHWeights = (nOptions x nHH) Household weights for grossing up shares
% X      = (structure)           contains data to compute shares
% Z     = (structure)     Copies of data to be interacted with random
%                            coefficients
%                            Z.z1 contains data on prices to be
%                            interacted with random coefficients
%
% Revision history
% 2017MAR19  Cleaned version for publication.

% Assume each (month,region) is a different market.

% Compute price inclusive of tax
if Tax.Type==1
  TaxPrice=repmat(Price+Tax.tax*Sats,[nHH 1]);
elseif Tax.Type==2
  TaxPrice=repmat(Price.*(1+Tax.tau*Sats),[nHH 1]);
elseif Tax.Type==3
  TaxPrice=repmat(Price,[nHH 1]);    
end

NewExpenditure = X.GrossExpenditure - TaxPrice;

% Insert new prices into data matrix
[X,Z]=UpdatePriceTerms(NewExpenditure,X,Z,basis,EstColumn,ZIndex, ...
                       iNonZero,RCDim);

% Compute implied market shares and gradient of market shares w.r.t. price

if nHH<=MaxN
  if nargout==1
    [S,GradS]=MarketShare2(X,Z,xFree2,iFree2,beta2,CDiag,COffDiag, ... 
                           LogNormalFlag,LogNormalIndex,EstColumn,ZIndex, ...
                           noptions,nHH,RCDim,RCRank,nFirms,  ...
                           AllFirmsProducts,NewHHProductCode, ...
                           Nodes,Weights,nQuadAll,Tax,Sats,   ...
                           AllCompetitorProducts);
    % Equilibrium:  Residual=0                  
    %  GradS(i,j,hh) = derivative of share j w.r.t. price i for hh
    S = repmat(S*HHWeights(1,:)',[1 noptions]);
    TempWeights = repmat(reshape(HHWeights,[1 noptions nHH]),[noptions 1 1]);
    GradS = sum(GradS.*TempWeights,3)./S;
    if basis.ExpendFlag==1
      GradS=-GradS;
    end
    
    % Profits depend on Price not on TaxPrice
    R = 1+GradS*(Price-Cost);

  elseif nargout==2
    % Additional outputs required to compute Jacobian
    [S,GradS,FullGradS,Hess]=MarketShare2(X,Z,xFree2,iFree2,beta2,CDiag,COffDiag, ... 
                              LogNormalFlag,LogNormalIndex,EstColumn,ZIndex,      ...
                              noptions,nHH,RCDim,RCRank,nFirms,  ...
                              AllFirmsProducts,NewHHProductCode, ...
                              Nodes,Weights,nQuadAll,Tax,Sats,AllCompetitorProducts);
    S           = repmat(S*HHWeights(1,:)',[1 noptions]);
    TempWeights = repmat(reshape(HHWeights,[1 noptions nHH]), ...
                         [noptions 1 1]);
    GradS       = sum(GradS.*TempWeights,3);
    FullGradS   = sum(FullGradS.*TempWeights,3);
    TempWeights = repmat(reshape(HHWeights,[1 1 noptions nHH]), ...
                         [noptions noptions 1 1]);
    % Hess(i,j,k,hh) derivative of share k w.r.t. price i then price j for
    %                hh
    Hess        = sum(Hess.*TempWeights,4);
    clear TempWeights;

    % Hess(i,j,k) = derivative of share k w.r.t. price i then price j
    Hess        = Hess./repmat(reshape(S(:,1),[noptions 1 1]), ...
                               [1 noptions noptions]);
                           
    GradS     = GradS./S;
    FullGradS = FullGradS./S';
    if basis.ExpendFlag==1
       GradS= - GradS;
       FullGradS = - FullGradS;
    end
    % Equilibrium:  Residual=0                  
    R = 1 + GradS*(Price-Cost);
    
    % Create copies of (Price-Cost) to multiply with Hess
    PriceCost = repmat(reshape(Price-Cost,[1 1 noptions]), ...
                       [noptions noptions 1]);
    
    % Multiply (Price-Cost) by Hess and then sum across dimension 3
    J         = GradS;
    % Create copies of GradS so that:
    % NewGradS(i,:,j) = GradS(i,j)
    GradS     = repmat(reshape(GradS,[noptions 1 noptions]),[1 noptions 1]);
    
    FullGradS = repmat(reshape(FullGradS',[noptions noptions 1]), ...
                       [1 1 noptions]);       
    J = J+sum((Hess-GradS.*FullGradS).*PriceCost,3);
  end  % if nargout==1

elseif nHH>MaxN
    
  S=zeros(noptions,1);
  GradS=zeros(noptions,noptions);

  if nargout==1
    parfor i1=1:ceil(nHH/MaxN)
    %for i1=1:ceil(nHH/MaxN)
      first = MaxN*noptions*(i1-1)+1;
      last  = min(MaxN*noptions*i1,nHH*noptions);
      index = (first:last)';
      nTemp=length(index)/noptions;

      TempZ = SelectZ(Z,index);
      TempX       = [];
      TempX.X     = X.X(index,:);
      TempX.GradX = X.GradX(index,:);
      TempX.HessX = X.HessX(index,:);
      TempX.Price = X.Price(index,:);
 
      if size(Nodes,1)>nQuadAll 
        TempNodes = CreateTempNodes(i1,MaxN,Nodes,nHH,nQuadAll,RCRank);
      else
        TempNodes = Nodes;
      end
      [TempS,TempGradS]=  ...
          Equilibrium2a(xFree2,iFree2,beta2,CDiag,COffDiag,    ...
          HHWeights(index),TempX,TempZ,LogNormalFlag,LogNormalIndex,EstColumn,ZIndex,  ...
          noptions,nTemp,RCDim,RCRank,nFirms,       ...
          AllFirmsProducts,NewHHProductCode(index), ...
          TempNodes,Weights,nQuadAll,Tax,Sats);
      if basis.ExpendFlag==1
        TempGradS = - TempGradS;
      end
      if nTemp>1
        S=S+sum(TempS,2);
        GradS=GradS+sum(TempGradS,3);
      elseif nTemp==1
        S=S+TempS;
        GradS=GradS+TempGradS;
      end
    end
  elseif nargout==2
    FullGradS=zeros(noptions,noptions);
    Hess=zeros(noptions,noptions,noptions);

    %???? PROBLEM IS HERE SOMEWHERE
    parfor i1=1:ceil(nHH/MaxN)
      first = MaxN*noptions*(i1-1)+1;
      last  = min(MaxN*noptions*i1,nHH*noptions);
      index = (first:last)';
      nTemp=length(index)/noptions;

      TempZ = SelectZ(Z,index);

      TempX       = [];
      TempX.X     = X.X(index,:);
      TempX.GradX = X.GradX(index,:);
      TempX.HessX = X.HessX(index,:);
    
      if size(Nodes,1)>nQuadAll
        TempNodes = CreateTempNodes(i1,MaxN,Nodes,nHH,nQuadAll,RCRank);
      else
        TempNodes = Nodes;
      end
      [TempS,TempGradS,TempFullGradS,TempHess]=  ...
         Equilibrium2a(xFree2,iFree2,beta2,CDiag,COffDiag,    ...
          HHWeights(index),TempX,TempZ,LogNormalFlag,LogNormalIndex,EstColumn,ZIndex, ...
          noptions,nTemp,RCDim,RCRank,nFirms,  ...
          AllFirmsProducts,NewHHProductCode(index), ...
          TempNodes,Weights,nQuadAll,Tax,Sats,AllCompetitorProducts);
      
      if basis.ExpendFlag==1
        % TempGradS = gradient w.r.t. (y-p)
        % so need to multiply by -1
        TempGradS     = -TempGradS;
        TempFullGradS = -TempFullGradS;
      end
      if nTemp>1
        S=S+sum(TempS,2);
        GradS=GradS+sum(TempGradS,3);
        FullGradS=FullGradS+sum(TempFullGradS,3);
        Hess=Hess+sum(TempHess,4);
      elseif nTemp==1
        S=S+TempS;
        GradS=GradS+TempGradS;
        FullGradS=FullGradS+TempFullGradS;
        Hess=Hess+TempHess;
      end
    end % parfor i1=1:ceil(nHH/MaxN)
    
    % Create copies of (Price-Cost) to multiply with Hess
    temp1=reshape(repmat((Price-Cost)',[noptions*noptions 1]),[noptions noptions noptions]);
  
    % Multiply (Price-Cost) by Hess and then sum across dimension 3
    temp2=sum(Hess.*temp1,3);
      J=FullGradS'+GradS+temp2;
  end
  
  % Equilibrium:  Residual=0
      R=S+GradS*(Price-Cost);
end   % if nHH<=MaxN