function [S,GradS]=MarketShare(X,Z,xFree,iFree,beta,CDiag,COffDiag, ... 
                               LogNormalFlag,LogNormalIndex,    ...
                               EstColumn,ZIndex, ...
                               noptions,nHH,RCDim,RCRank,  ...
                               nFirms,AllFirmsProducts,HHProductCode, ...
                               Nodes,Weights,nQuad)

% Compute market shares and gradient of market shares w.r.t. expenditure for
% mixed logit model
%
%
%
% S	            =  (noptions x nHH) vector of market shares
% GradS         =  (noptions x noptions x nHH) Gradient of market share w.r.t. price
% xFree         =  (nFree x 1) vector of free parameters
% iFree{1}      =  (n1 x 1) indexes in beta corresponding to free parameters
% iFree{2}      =  (n2 x 1) indexes in xFree corresponding to beta parameters
% iFree{3}      =  (n3 x 1) indexes in C corresponding to free parameters
% iFree{4}      =  (n4 x 1) indexes in xFree corresponding to C parameters
% beta          =  (nx    x 1) all parameters in x
% C
% X.X        =  (noptions*nHH x k) data matrix (sparse)
% X.GradX    =  (noptions*nHH x k) Used to calculate gradient w.r.t.
%                                      price.
% Z.Z          =  (noptions*nHH x RCDim) data matrix to interact with
%                                          random coefficients
% EstColumn.Price   = (1 x 1) estimation data column of price
% nHH           =  (1 x 1) number of observations
% noptions      =  (1 x 1) number of options
% RCDim         =  (1 x 1) dimension of vector of random coefficients
% Nodes         =  (nQuad x RCDim) nodes for numerical integration
% Weights       =  (nQuad x 1) weights for numerical integration
% nQuad         =  (1 x 1) number of integration nodes
%
% Revision history
% 2017MAR18   Cleaned version for publication.

big=600;

% map xFree to (beta,C)
[beta,C]=MapFreeToStructural(xFree,iFree,beta,CDiag,COffDiag,RCDim);
C = C(:,1:RCRank);

% v1 = (J*N x nQuad) utility obtained from each product (including random coefficients)
% GradV1 = (J*N x nQuad)  gradient of v1 w.r.t. price
v1=repmat(X.X*beta,[1 nQuad]);
GradV1=full(repmat(X.GradX*beta(EstColumn.Expenditure),[1 nQuad]));

RCMethod=2; % 1: uses a lot of memory
            % 2: uses less memory but with an added loop
            
% Random coefficients
% size(e) = (nQuad x RCDim)
e=Nodes*C.';
if LogNormalFlag==1
  e(:,LogNormalIndex) = -exp(e(:,LogNormalIndex));
end

if RCMethod==1
  for i1=1:RCDim
    % size(e1) = (noptions*nHH x nQuad)
    if (size(e,1)==nQuad)
      % same draws for all households, must replicate
      e1 = repmat(e(:,i1),[1 noptions*nHH]).';
    else
      % different draws for each HH
      nHHQuad = size(e,1)/nQuad;
      nT      = nHH/nHHQuad;
      e1=kron(reshape(e(:,i1),[nHHQuad nQuad]),ones(nT*noptions,1));
    end

    % size(v1) = (noptions*nHH x nQuad)
    % size(Z.Z(:,i1)) = (noptions*nHH x 1)
    v1 = v1 + e1.*repmat(Z.Z(:,i1),[1 nQuad]);
    
    % if Z.GradZ(:,i1) depends on price, compute price impact
    if any(i1==ZIndex.iP)
      for i2=1:nQuad
        GradV1(:,i2) = GradV1(:,i2) + e1(:,i2).*Z.GradZ(:,i1);
      end
    end
  end   % for i1=1:RCDim
  clear e1;
elseif RCMethod==2
  for i1=1:RCDim
  for i2=1:nQuad
    if (size(e,1)==nQuad)
      v1(:,i2) = v1(:,i2) + e(i2,i1)*Z.Z(:,i1);
      if any(i1==ZIndex.iP)
        GradV1(:,i2) = GradV1(:,i2)+e(i2,i1)*Z.GradZ(:,i1);
      end
    else
      % different draws for each HH
      nHHQuad = size(e,1)/nQuad;
      nT = nHH/nHHQuad;
      e1 = kron(e((i2-1)*nHHQuad+(1:nHHQuad)',i1),ones(nT*noptions,1));
      v1(:,i2) = v1(:,i2) + e1.*Z.Z(:,i1);
      if any(i1==ZIndex.iP)
        GradV1(:,i2) = GradV1(:,i2) + e1.*Z.GradZ(:,i1);
      end
    end  % if (size(e,1)==nQuad)
  end    % for i2=1:nQuad
  end    % for i1=1:RCDim
end      % if RCMethod==1

v1 = reshape(v1,[noptions nHH nQuad]);
MaxV1 = max(v1,[],1);
v1 = reshape(v1 - repmat(MaxV1,[noptions 1 1]),[noptions*nHH nQuad]);
v1=exp(min(v1,big));

% Sum across options, replicate and reshape to obtain denom with:
% size(denom) = (noptions*nHH x nQuad)
denom=sum(reshape(v1,noptions,nHH,nQuad),1);
denom=repmat(denom,[noptions 1 1]);
denom=reshape(denom,noptions*nHH,nQuad);
  
% Probability of choice average across epsilon
% size(prob0)  = (noptions*nHH x nQuad)
% size(prob)   = (noptions x nHH)
prob0=v1./denom;                           % probability conditional on epsilon
prob=reshape(prob0*Weights,noptions,nHH);  % HH specific probability averaged across epsilon

% sum across households to compute market shares.
% size(S) = (noptions x nHH)
S=prob;

% Calculate gradient of S w.r.t. to own prices
GradS=zeros(noptions,noptions,nHH);

%Price=reshape(X(:,PriceIndex1),noptions,nHH);

for f=1:nFirms
  % Indexes of Products owned by firm f
  CurrentProducts=AllFirmsProducts{f};
  % number of products owned by firm f
  nProducts=length(CurrentProducts);
  for j1=1:nProducts
    % Gradient of shares w.r.t. own price  
    % Pick out rows of prob0 and Gradv1 corresponding to CurrentProducts(j1)
    % Then compute weighted average across epsilon using Weights
    % size(temp) = (nHH x 1)
    temp=(prob0(HHProductCode==CurrentProducts(j1),:) ...
          .*(1-prob0(HHProductCode==CurrentProducts(j1),:)) ...
          .*GradV1(HHProductCode==CurrentProducts(j1),:))*Weights;
    GradS(CurrentProducts(j1),CurrentProducts(j1),:)=reshape(temp,[1 1 nHH]);

    for j2=1:nProducts
      % In general, this need not be symmetric
      % Gradient of shares w.r.t. to cross-prices (within firm only)
      % Pick out rows of prob0 corresponding to products
      % CurrentProducts(j1) and CurrentProducts(j2)
      if j2~=j1
        temp=-(prob0(HHProductCode==CurrentProducts(j1),:)   ...
               .*prob0(HHProductCode==CurrentProducts(j2),:) ...
               .*GradV1(HHProductCode==CurrentProducts(j2),:))*Weights;
        GradS(CurrentProducts(j2),CurrentProducts(j1),:)=reshape(temp,[1 1 nHH]);
      end
    end
  end    
end
