function [S,GradS,FullGradS,Hess] = ...
    MarketShare2(X,Z,xFree,iFree,beta,CDiag,COffDiag,      ... 
                 LogNormalFlag,LogNormalIndex,EstColumn,ZIndex, ...
                 noptions,nHH,RCDim,RCRank,nFirms,      ...
                 AllFirmsProducts,HHProductCode, ...
                 Nodes,Weights,nQuad,Tax,Sats,   ...
                 AllCompetitorProducts)

%
% Compute market shares and gradient of market shares w.r.t. price for
% mixed logit model.
%   (S,GradS) are the same as MarketShare.
%   (FullGradS,Hess) are additional outputs computed by MarketShare2
%
% FullGradS(i,j) contains derivative of share j w.r.t. to price i for all
%                (i,j)
%
% GradS(i,j)  contains derivative of share j w.r.t. to price i if (i,j) are 
%             owned by the same firm.
%             otherwise equals zero.
% Hess(i,j,k,h) contains partial derivative of S(k) w.r.t. p(i) and p(j)
%               for household h.
%               if BOTH i and j are NOT owned by firm f(k), Hess(i,j,k,h)=0. 
% 
% Treat each household as a separate market.
% 
%
% S	            =  (noptions x nHH) vector of market shares
% GradS         =  (noptions x noptions x nHH) Gradient of market share
%                  w.r.t. price. (w.r.t. other firms' prices set to zero)
% FullGradS     =  (noptions x noptions x nHH) Gradient of market share
%                  w.r.t. price
% Hess          =  (noptions x noptions x noptions x nHH) Derivative of
%                  GradS w.r.t. price
% xFree         =  (nFree x 1) vector of free parameters
% iFree{1}      =  (n1 x 1) indexes in beta corresponding to free parameters
% iFree{2}      =  (n2 x 1) indexes in xFree corresponding to beta parameters
% iFree{3}      =  (n3 x 1) indexes in C corresponding to free parameters
% iFree{4}      =  (n4 x 1) indexes in xFree corresponding to C parameters
% beta          =  (nx    x 1) all parameters in x
% C
% X.X          =  (noptions*nHH x k) data matrix (sparse)
% X.GradX      =  (noptions*nHH x k) Used to calculate gradient w.r.t.
%                                      price.
% Z          =  (noptions*nHH x RCDim) data matrix to interact with
%                                          random coefficients
% nHH          =  (1 x 1) number of observations
% noptions      =  (1 x 1) number of options
% RCDim         =  (1 x 1) dimension of vector of random coefficients
% Nodes         =  (nQuad x RCDim) nodes for numerical integration
% Weights       =  (nQuad x 1) weights for numerical integration
% nQuad         =  (1 x 1) number of integration nodes
%
%
% Revision history
% 2017MAR18   Cleaned version for publication.

% transform xFree into (beta,C)
[beta,C]=MapFreeToStructural(xFree,iFree,beta,CDiag,COffDiag,RCDim);
C = C(:,1:RCRank);

v1=repmat(X.X*beta,[1 nQuad]);
GradV1=repmat(X.GradX*beta(EstColumn.Expenditure),[1 nQuad]);
if nargout>2
  HessV1=repmat(X.HessX*beta(EstColumn.Expenditure),[1 nQuad]);
end

% Random coefficients
e=Nodes*C.';
if LogNormalFlag==1
  e(:,LogNormalIndex) = -exp(e(:,LogNormalIndex));
end

RCMethod = 1;  % 1 : uses a lot of memory
               % 2 : uses much less memory but with a loop

if RCMethod==1
  for i1=1:RCDim
    % size(e1) = (noptions*nHH x nQuad)
    if (size(e,1)==nQuad)
      % same draws for all households, must replicate
      e1 = repmat(e(:,i1),[1 noptions*nHH]).';
    else
      % different draws for each HH
      nHHQuad = size(e,1)/nQuad;
      nT      = nHH/nHHQuad;
      e1=kron(reshape(e(:,i1),[nHHQuad nQuad]),ones(nT*noptions,1));
    end
    
    % size(v1) = (noptions*nHH x nQuad)
    % size(Z.Z(:,i1)) = (noptions*nHH x 1)
    v1 = v1 + e1.*repmat(Z.Z(:,i1),[1 nQuad]);
    if any(i1==ZIndex.iP)
      for i2=1:nQuad
        GradV1(:,i2) = GradV1(:,i2) + e1(:,i2).*Z.GradZ(:,i1);
        if nargout>2
          HessV1(:,i2) = HessV1(:,i2)+ e1(:,i2).*Z.HessZ(:,i1);
        end
      end
    end
  end
  clear e1;
elseif RCMethod==2
  for i1=1:RCDim
  for i2=1:nQuad
    if (size(e,1)==nQuad)  
      v1(:,i2) = v1(:,i2) + e(i2,i1)*Z.Z(:,i1);
      if any(i1==ZIndex.iP)
        GradV1(:,i2) = GradV1(:,i2) + e(i2,i1)*Z.GradZ(:,i1);
        if nargout>2
          HessV1(:,i2) = HessV1(:,i2) + e(i2,i1)*Z.HessZ(:,i1);
        end
      end
    else
      % different draws for each HH
      nHHQuad = size(e,1)/nQuad;
      nT = nHH/nHHQuad;
      e1 = kron(e((i2-1)*nHHQuad+(1:nHHQuad)',i1),ones(nT*noptions,1));
      v1(:,i2) = v1(:,i2) + e1.*Z.Z(:,i1);
      if any(i1==ZIndex.iP)
        GradV1(:,i2) = GradV1(:,i2) + e1.*Z.GradZ(:,i1);
        if nargout>2
          HessV1(:,i2)=HessV1(:,i2) + e1.*Z.HessZ(:,i1);
        end
      end
    end   % if size(e,1)==nQuad
  end   % for i2=1:nQuad
  end   % for i1=1:RCDim
end     % if RCMethod==1

v1 = reshape(v1,[noptions nHH nQuad]);
MaxV1 = max(v1,[],1);
v1 = reshape(v1 - repmat(MaxV1,[noptions 1 1]),[noptions*nHH nQuad]);
v1=exp(v1);

% Sum across options, replicate and reshape to obtain denom with:
% size(denom) = (noptions*nHH x nQuad)
denom=sum(reshape(v1,noptions,nHH,nQuad),1);
denom=repmat(denom,[noptions 1 1]);
denom=reshape(denom,noptions*nHH,nQuad);
  
% Probability of choice average across epsilon
% size(prob0)  = (noptions*nHH x nQuad)
% size(prob)   = (noptions x nHH)

% prob0 = probability conditional on epsilon
prob0=v1./denom;

clear v1 denom;

% sum across households to compute market shares.
% size(S) = (noptions x nHH)
S=reshape(prob0*Weights,noptions,nHH);    % probability averaged across epsilon

% Calculate gradient of S w.r.t. to own prices
GradS=zeros(noptions,noptions,nHH);
if nargout>2
  FullGradS=zeros(noptions,noptions,nHH);
  Hess=zeros(noptions,noptions,noptions,nHH);
end

for f=1:nFirms
    
  % Indexes of Products owned by firm f
  CurrentProducts=AllFirmsProducts{f};
  
  % number of products owned by firm f
  nProducts=length(CurrentProducts);
  
  if nargout>2
    % Indexes of products NOT owned by firm f
    CompetitorProducts=AllCompetitorProducts{f};

    % number owned by competitors
    nCompete=noptions-nProducts;
  end
 
  for j1=1:nProducts
    % Gradient of shares w.r.t. own price  
    % Pick out rows of prob0 and GradV1 corresponding to CurrentProducts(j1)
    % Then compute weighted average across epsilon using Weights
    % size(J1Rows) = (nHH*noptions x 1)
    % size(prob0)  = (noptions*nHH x nQuad)
    % size(temp) = (nHH x 1)
    % J1Rows = index of (hh,product) observations corresponding to
    %            product j1
    J1Rows = HHProductCode==CurrentProducts(j1);
    temp = (prob0(J1Rows,:).*(1-prob0(J1Rows,:)).*GradV1(J1Rows,:)) ...
           *Weights;
    
    if Tax.Type==2
      temp=temp*(1+Tax.tau*Sats(CurrentProducts(j1)));
    end
    GradS(CurrentProducts(j1),CurrentProducts(j1),:)=reshape(temp,[1 1 nHH]);
  
    if nargout>2
      % Fix j1 and loop through all j2:
      %    Derivative of s(j1) w.r.t. p(j1) and p(j2)
      for j2=1:noptions
        J2Rows = HHProductCode==j2;
        if j2==CurrentProducts(j1)  
          % Derivative of s(j1) w.r.t. p(j1) and p(j1)  
          %    prob0(j1).*(1-prob0(j1)) .*
          %        (HessV1(j1,:) + (1-2*prob0(j1,:)).*GradV1(j1,:))
          %  * Weights
          temp = (prob0(J1Rows,:).*(1-prob0(J1Rows,:)) ...
                 .*(HessV1(J1Rows,:)+(1-2*prob0(J1Rows,:)) ...
                                       .*GradV1(J1Rows,:)    ...
                                       .*GradV1(J1Rows,:)))  ...
                 *Weights;
           
           if Tax.Type==2
             temp=temp.*(1+Tax.tau*Sats(CurrentProducts(j1))).^2;
           end

           % Hess(i,j,k,h) contains partial derivative of S(k) w.r.t. p(i) and p(j)
           %               for household h.
           %               if BOTH i and j are NOT owned by firm f(k), Hess(i,j,k,h)=0. 
          Hess(CurrentProducts(j1),CurrentProducts(j1),CurrentProducts(j1),:) = ...
               temp;
        else
          % Derivative of s(j1) w.r.t. p(j1) and p(j2)   (j2~=j1)  
          temp = -(prob0(J1Rows,:).*prob0(J2Rows,:) ...
                   .*(1-2*prob0(J1Rows,:)) ...
                   .*GradV1(J1Rows,:)      ...
                   .*GradV1(J2Rows,:))*Weights;
          if Tax.Type==2
            temp=temp*(1+Tax.tau*Sats(CurrentProducts(j1))) ...
                     *(1+Tax.tau*Sats(j2));
          end
          Hess(CurrentProducts(j1),j2,CurrentProducts(j1),:)= ...
               temp;
        end % if j2==CurrentProducts(j1)
      end   % for j2=1:noptions
    end     % if nargout>2
    
    % Gradient of shares w.r.t. to cross-prices (within firm only)
    % i.e. gradient of share j1 w.r.t. to price j2, both (j1,j2) owned by f
    % 
    % In general, this need not be symmetric
    % Pick out rows of prob0 corresponding to products
    % CurrentProducts(j1) and CurrentProducts(j2)
    for j2=1:nProducts
      J2Rows = HHProductCode == CurrentProducts(j2);
      if j2~=j1
        % Gradient of share j1 w.r.t. price j2
        % Both j1 and j2 owned by firm f
        TempJ1J2 = prob0(J1Rows,:) ...
                     .*prob0(J2Rows,:);
        temp=-(TempJ1J2.*GradV1(J2Rows,:))*Weights;
        if Tax.Type==2
          temp=temp*(1+Tax.tau*Sats(CurrentProducts(j2)));
        end
        GradS(CurrentProducts(j2),CurrentProducts(j1),:)=reshape(temp,[1 1 nHH]);
        
        if nargout>2
          for j3=1:noptions
            % Hess(i,j,k,h) contains partial derivative of S(k) w.r.t. p(i) and p(j)
            %               for household h.
            %               if BOTH i and j are NOT owned by firm f(k), Hess(i,j,k,h)=0. 

            % Derivative of share i w.r.t. prices j2 and j3
            if j3==CurrentProducts(j1)
              % derivative of s(j1) w.r.t. p(j2) and p(j1)
              Hess(CurrentProducts(j2),j3,CurrentProducts(j1),:)=...
                   Hess(j3,CurrentProducts(j2),CurrentProducts(j1),:);
            elseif CurrentProducts(j2)==j3
              % derivative of s(j1) w.r.t. p(j2) and p(j2)  
              temp = -(prob0(J1Rows,:).*prob0(J2Rows,:) ...
                       .*(HessV1(J2Rows,:)+(1-2*prob0(J2Rows,:)) ...
                                           .*GradV1(J2Rows,:)    ...
                                           .*GradV1(J2Rows,:)))   ...
                      *Weights;
              if Tax.Type==2
                temp=temp*(1+Tax.tau*Sats(CurrentProducts(j2))) ...
                         *(1+Tax.tau*Sats(CurrentProducts(j2)));
              end
                
              Hess(j3,j3,CurrentProducts(j1),:)= ...
                   temp;
            else
              % derivative of s(j1) w.r.t. p(j2) and p(j3)
              temp=2*(prob0(J1Rows,:).*prob0(J2Rows,:) ...
                     .*prob0(HHProductCode==j3,:) ...
                     .*GradV1(J2Rows,:) ...
                     .*GradV1(HHProductCode==j3,:))*Weights;
              if Tax.Type==2
                temp=temp*(1+Tax.tau*Sats(CurrentProducts(j2))) ...
                         *(1+Tax.tau*Sats(j3));
              end
              Hess(CurrentProducts(j2),j3,CurrentProducts(j1),:)= ...
                   temp;
            end
          end  % for j3=1:noptions
        end    % if nargout>2
      end      % if j2~=j1
    end        % for j2=1:nProducts
  
    if nargout>2
      % Gradient of own products w.r.t. competitor prices
      % j1 is a product owned by firm f
      % j2 is a product NOT owned by firm f
      for j2=1:nCompete
        temp=-(prob0(J1Rows,:)   ...
               .*prob0(HHProductCode==CompetitorProducts(j2),:) ...
               .*GradV1(HHProductCode==CompetitorProducts(j2),:))*Weights;
        if Tax.Type==2
          temp=temp*(1+Tax.tau*Sats(CompetitorProducts(j2)));
        end
        FullGradS(CompetitorProducts(j2),CurrentProducts(j1),:)=reshape(temp,[1 1 nHH]);
      end
    end % if nargout>2
  end   % for j1 = 1:nProducts    
end     % for f  = 1:nFirms

if nargout>2
  FullGradS=FullGradS+GradS;
end