function [b,db,ddb]=bspline5(x,knots,k,sorted,knots0,knots1)
% [b,db,ddb]=bspline5(x,knots,k,sorted,knots0,knots1)
%
% Compute B-splines of degree k (order k+1) at the vector of points x
% Also compute db, the derivatives of the B-splines and ddb, the second derivatives of the B-splines.
% 
% b      = (nknots-1+k x nx)   b(i,j) =  B^k_i(x(j))
% db	 = (nknots-1+k x nx)   db(i,j) = dB^k_i(x(j))/dx
% ddb	 = (nknots-1+k x nx)   ddb(i,j) = d^2B^k_i(x(j))/dx*dx
% x      = (nx x 1)            vector of points
% knots  = (nknots x 1)        knots for spline
% k      = (1 x 1)             degree of B-spline
% sorted = (1 x 1)             1 if sorted, 0 otherwise
% knots0 = (k x 1)             extra knots below knots(1, to define endpoint behaviour
% knots1 = (k x 1)             extra knots above knots(n), to define endpoint behaviour
%
%       if k==3, the splines are cubic splines
%  
% Version "bspline5.m" searches each interval in grid in ascending order, vector processing.
% That is it computes the columns of b, db and ddb in batches computing values for several elements of x
% simultaneously.
% This program is identical to "bspline3.m" and "bspline4.m" except that this version computes ddb in addition to b and db.
%
% September 12, 2006
% 	I tested the accuracy of the derivative calculations (both 1st and 2nd derivatives) against numerical derivatives.
%	The derivatives are accurate up to machine precision. 

if nargin<3, k=3; end;
if nargin<4, sorted=0; end

if sorted==0
  [x,i00]=sort(x);
  [~,i02]=sort(i00);
end


nx=length(x);
nknots=length(knots);

if nargin<6, 
  knots0=knots(1)*ones(k,1);
  knots1=knots(nknots)*ones(k,1);
end


n2=nknots+k-1;
b=zeros(n2,nx);
db=b;
ddb=db;

knots=[knots0;knots;knots1];

i0=(1:nx)';
x2=ones(n2,1);
for j1=k+2:nknots+k
  if isempty(i0), break, end
  i1=find(x(i0)<=knots(j1));
  if ~isempty(i1)
    n1=length(i1);
    x1=ones(1,n1);
    b(j1-1,i0(i1))=x1;
    for j2=1:k
      t1d=(knots(1+j2:j2+n2)-knots(1:n2))*x1;    
      t2d=(knots(2+j2:j2+n2+1)-knots(2:n2+1))*x1;
      t1n=(x2*x(i0(i1))'-knots(1:n2)*x1);
      t2n=(knots(2+j2:j2+n2+1)*x1-x2*x(i0(i1))');

      t1=zeros(n2*n1,1);
      t2=t1;
      t1d=t1d(:);
      t2d=t2d(:);
      
      if any(t1d==0) || any(t2d==0)
        i1d=find(t1d~=0);
        i2d=find(t2d~=0);
        if j2>(k-2)
          t4=t1;
          t5=t1;
        end
        if ~isempty(i1d)
          t1(i1d)=t1n(i1d)./t1d(i1d);
          if j2==(k-1)
            t4(i1d)=(k-1)./t1d(i1d);
            t4=reshape(t4,n2,n1);
          elseif j2==k
            t4(i1d)=k./t1d(i1d);
            t4=reshape(t4,n2,n1);
          end
        end
        if ~isempty(i2d)
          t2(i2d)=t2n(i2d)./t2d(i2d);
          if j2==(k-1)
            t5(i2d)=-(k-1)./t2d(i2d);
            t5=reshape(t5,n2,n1);
          elseif j2==k
            t5(i2d)=-k./t2d(i2d);
            t5=reshape(t5,n2,n1);
          end
        end
        t1=reshape(t1,n2,n1);
        t2=reshape(t2,n2,n1);

        if j2==(k-1)
          ddb(:,i0(i1))=t4.*b(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
        elseif j2==k
          db(:,i0(i1))=t4.*b(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
          ddb(:,i0(i1))=t4.*ddb(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[ddb(2:n2,i0(i1));zeros(1,n1)];
        end
      else
        t1=(x2*x(i0(i1))'-knots(1:n2)*x1)./t1d;
        t2=(knots(2+j2:j2+n2+1)*x1-x2*x(i0(i1))')./t2d;
        if j2==(k-1)
          t4=(k-1)./t1d;
          t5=-(k-1)./t2d;
          ddb(:,i0(i1))=t4.*b(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
        elseif j2==k
          t4=k./t1d;
          t5=-k./t2d;
          db(:,i0(i1))=t4.*b(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
          ddb(:,i0(i1))=t4.*ddb(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[ddb(2:n2,i0(i1));zeros(1,n1)];
        end
      end
      b(:,i0(i1))=t1.*b(:,i0(i1))+[t2(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
    end
  end
%  i2=find(x(i0)>knots(j1));
%  i0=i0(i2);
  i0 = i0(x(i0)>knots(j1));
end

if ~isempty(i0)
  i1=find(x(i0)>knots(j1));
  if ~isempty(i1)
    n1=length(i1);
    x1=ones(1,n1);
    b(j1-1,i0(i1))=x1;
    for j2=1:k
      t1d=(knots(1+j2:j2+n2)-knots(1:n2))*x1;    
      t2d=(knots(2+j2:j2+n2+1)-knots(2:n2+1))*x1;
      t1n=(x2*x(i0(i1))'-knots(1:n2)*x1);
      t2n=(knots(2+j2:j2+n2+1)*x1-x2*x(i0(i1))');

      t1=zeros(n2*n1,1);
      t2=t1;
      t1d=t1d(:);
      t2d=t2d(:);
      
      if any(t1d==0) || any(t2d==0)
        i1d=find(t1d~=0);
        i2d=find(t2d~=0);
        if j2>=(k-1)
          t4=t1;
          t5=t1;
        end
        if ~isempty(i1d)
          t1(i1d)=t1n(i1d)./t1d(i1d);
          if j2==(k-1)
            t4(i1d)=(k-1)./t1d(i1d);
            t4=reshape(t4,n2,n1);
          elseif j2==k
            t4(i1d)=k./t1d(i1d);
            t4=reshape(t4,n2,n1);
          end
        end
        if ~isempty(i2d)
          t2(i2d)=t2n(i2d)./t2d(i2d);
          if j2==(k-1)
            t5(i2d)=-(k-1)./t2d(i2d);
            t5=reshape(t5,n2,n1);
          elseif j2==k
            t5(i2d)=-k./t2d(i2d);
            t5=reshape(t5,n2,n1);
          end
        end
        t1=reshape(t1,n2,n1);
        t2=reshape(t2,n2,n1);

        if j2==(k-1)
          ddb(:,i0(i1))=t4.*b(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
        elseif j2==k
          db(:,i0(i1))=t4.*b(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
          ddb(:,i0(i1))=t4.*ddb(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[ddb(2:n2,i0(i1));zeros(1,n1)];
        end
      else
        t1=(x2*x(i0(i1))'-knots(1:n2)*x1)./t1d;
        t2=(knots(2+j2:j2+n2+1)*x1-x2*x(i0(i1))')./t2d;
        if j2==(k-1)
          t4=(k-1)./t1d;
          t5=-(k-1)./t2d;
          ddb(:,i0(i1))=t4.*b(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
        elseif j2==k
          t4=k./t1d;
          t5=-k./t2d;
          db(:,i0(i1))=t4.*b(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
          ddb(:,i0(i1))=t4.*ddb(:,i0(i1))+[t5(1:n2-1,:);zeros(1,n1)].*[ddb(2:n2,i0(i1));zeros(1,n1)];
        end
      end
      b(:,i0(i1))=t1.*b(:,i0(i1))+[t2(1:n2-1,:);zeros(1,n1)].*[b(2:n2,i0(i1));zeros(1,n1)];
    end
  end
end

if sorted==0
  b=b(:,i02);
  db=db(:,i02);
  ddb=ddb(:,i02);
end
