function [V1,GradV1,MaxV1,V10]=ComputeExpUtility(xFree,iFree,beta,CDiag,COffDiag, ...
                                                 X,Z,LogNormalFlag,LogNormalIndex, ...
                                                 EstColumn,ZIndex,noptions,nHH,    ...
                                                 RCDim,RCRank,Nodes,nQuad)
% Compute expected utility.
%
%
% xFree         =  (nFree x 1) vector of free parameters
% iFree{1}      =  (n1 x 1) indexes in beta corresponding to free parameters
% iFree{2}      =  (n2 x 1) indexes in xFree corresponding to beta parameters
% iFree{3}      =  (n3 x 1) indexes in C corresponding to free parameters
% iFree{4}      =  (n4 x 1) indexes in xFree corresponding to C parameters
% beta          =  (nx    x 1) all parameters in x
% CDiag
% COffDiag
% X             =  (noptions*nHH x k) data matrix (sparse)
% GradX         =  (noptions*nHH x k) Used to calculate gradient w.r.t.
%                                      price.
% Z             =  (noptions*nHH x RCDim) data matrix to interact with
%                                          random coefficients
% nHH           =  (1 x 1) number of observations
% noptions      =  (1 x 1) number of options
% RCDim         =  (1 x 1) dimension of vector of random coefficients
% Nodes         =  (nQuad x RCDim) nodes for numerical integration
% nQuad         =  (1 x 1) number of integration nodes
%
%
% Revision history
% 2017MAR19 Cleaned version for publication.


[beta,C]=MapFreeToStructural(xFree,iFree,beta,CDiag,COffDiag,RCDim);
C = C(:,1:RCRank);

% V1     = (J*N x nQuad) utility obtained from each product (including random coefficients)
% GradV1 = (J*N x nQuad) gradient of V1 w.r.t. price
V1=repmat(X.X*beta,[1 nQuad]);

if nargout>1
  GradV1=full(repmat(X.GradX*beta(EstColumn.Expenditure),[1 nQuad]));
else
  GradV1=0;
end

RCMethod=2;  % 1 uses a lot of memory
             % 2 uses less memory but with an added loop

% Random coefficients
e=Nodes*C.';
if LogNormalFlag==1
  e(:,LogNormalIndex) = -exp(e(:,LogNormalIndex));
end

if RCMethod==1
  for i1=1:RCDim
    % size(e1) = (noptions*nHH x nQuad)
    if (size(e,1)==nQuad)
      % same draws for all households, must replicate e
      e1 = repmat(e(:,i1),[1 noptions*nHH]).';
    else
      % different draws for each HH
      nHHQuad = size(e,1)/nQuad;
      nT = nHH/nHHQuad;
      e1=kron(reshape(e(:,i1),[nHHQuad nQuad]),ones(nT*noptions,1));
    end

    % size(V1) = (noptions*nHH x nQuad)
    % size(Z.Z) = (noptions*nHH x RCDim)
    V1 = V1 + e1.*repmat(Z.Z(:,i1),[1 nQuad]);
    
    % if Z.GradZ(:,i1) depends on price, compute price impact
    if nargout>1 && any(i1==ZIndex.iP)
      for i2=1:nQuad
        GradV1(:,i2) = GradV1(:,i2) + e1(:,i2).*Z.GradZ(:,i1);
      end
    end
  end   % for i1=1:RCDim
elseif RCMethod==2
  for i1=1:RCDim
  for i2=1:nQuad
    if (size(e,1)==nQuad)
      V1(:,i2) = V1(:,i2) + e(i2,i1)*Z.Z(:,i1);
      if nargout>1 && any(i1==ZIndex.iP)
        GradV1(:,i2) = GradV1(:,i2) + e(i2,i1)*Z.GradZ(:,i1);
      end
    else
      % different draws for each HH
      nHHQuad = size(e,1)/nQuad;
      nT =nHH/nHHQuad;
      e1 = kron(e((i2-1)*nHHQuad+(1:nHHQuad)',i1),ones(nT*noptions,1));
      V1(:,i2) = V1(:,i2) + e1.*Z.Z(:,i1);
      if nargout>1 && any(i1==ZIndex.iP)
        GradV1(:,i2) = GradV1(:,i2) + e1.*Z.GradZ(:,i1);
      end
    end
  end  % for i2=1:nQuad
  end  % for i1=1:RCDim
end     % if RCMethod==1


% Above:
% size(V1)     = (J*N x nQuad)
% size(GradV1) = (J*N x nQuad)
%
% Now:
% size(V10)    = (noptions x nHH x nQuadAll)
% size(V1)     = (1 x nHH x nQuadAll)
% size(GradV1) = (nHH x nQuadAll)
V10 = reshape(V1,[noptions nHH nQuad]);
MaxV1 = max(V10,[],1);
V1  = sum(exp(V10-repmat(MaxV1,[noptions 1 1])),1);
S1  = exp(V10-repmat(MaxV1,[noptions 1 1]))./repmat(V1,[noptions 1 1]);
V1  = reshape(V1,nHH,nQuad);
MaxV1 = reshape(MaxV1,[nHH nQuad]);

% In linear utility case, gradient is simply slope w.r.t. price
% In non-linear case, gradient is share weighted average or slope w.r.t.
%     price
if nargout>1
  GradV1 = -reshape(sum(S1.*reshape(GradV1,noptions,nHH,nQuad),1),[nHH nQuad]);
end