function WelfareResults = ...
    ComputeWelfare_Dagsvik(xFree2,iFree2,beta2,CDiag,    ...
                           COffDiag,HHID,                ... 
                           HHPrice,HHTaxPrice,           ...
                           HHNashConsumerPrice,          ...
                           LogNormalFlag,LogNormalIndex, ...
                           basis,EstColumn,ZIndex,       ...
                           noptions,nHH,RCDim,RCRank, ...
                           TempDir,Nodes,Weights,nQuadAll,brep)

% function WelfareResults = ComputeWelfare_Dagsvik(BaseTaxDataFile,HHTaxResultsFile)
%
% Compute ex-post compensating variation using vormula from Dagsvik and Karlstrom (2005):
%     E(CV) =  E(   U(z,p0,eps) ) = E( U(z,p1-CV,eps) )
%
%   1) For each (HH,j,beta), compute u(z,y-p0(j),j,beta)
%   2) for each (HH,j,beta), find y1(j) to solve
%        u(z,y0-p0(j),j,beta) = u(z,y1(j)-p1(j),j,beta)
%   3) for each (HH,j,beta), compute integral over s from 0 to y1(j) 
%      1 / sum_{k} \exp{max{u(z,s-p1(k),k,beta),u(z,y0-p0(k),k,beta)} 
%   4) for each (HH,beta) compute weighted sum over j of 3) with weights
%       exp(u(z,y0-p0(j),j,beta))
%   5) For each HH, compute average over beta
%
% See Dagsvik and Karlstrom (2005)
%
% WelfareResults  = (nHH x 3) =  [HHID CV_Tax CV_Nash]
%
% Revision history
% 2017MAR19  Cleaned version for publication.
StartTime=clock;
DisplayFlag=0;

%% Compute base utility
% load base values of (X,Z,GradX,HessX)
load(fullfile(TempDir,'X.mat'));
CurrentTime=clock;
if DisplayFlag==1
  disp(['Current time: ',int2str(CurrentTime(4)),':',int2str(CurrentTime(5))]);
  disp('Computing base utility.')
end
  tic;
  
  % size(V0) = (noptions x nHH x nQuadAll)
  V0=ComputeUtility_all(xFree2,iFree2,beta2,CDiag,COffDiag, ...
                        X,Z,LogNormalFlag,LogNormalIndex,   ...
                        noptions,nHH,RCDim,RCRank,Nodes,nQuadAll);
  t1=toc/60;
  CurrentTime=clock;
if DisplayFlag==1
  disp(['Base utility computation time: ',int2str(t1),' minutes.']);
  disp(['Elapsed time: ',num2str(etime(CurrentTime,StartTime)/60),' minutes.']);
end
%% Compute welfare impact of tax increase: NO FIRM RESPONSE
% Replace HHPrice with HHTaxPrice

iNonZero = HHPrice~=0;
% size(HHTaxPrice) = (nHH*noptions x 1)

if DisplayFlag==1
  disp(['Current time: ',int2str(CurrentTime(4)),':',int2str(CurrentTime(5))]);
  disp('Computing No Firm Response utility.')
end
tic;
if brep==0
  CV_Tax = ComputeCV_Dagsvik(V0,HHTaxPrice,X,Z,basis,          ...
                             EstColumn,ZIndex,iNonZero,          ...
                             xFree2,iFree2,beta2,CDiag,COffDiag, ...
                             LogNormalFlag,LogNormalIndex,       ...
                             noptions,nHH,RCDim,RCRank,nQuadAll, ...
                             Nodes,Weights);
else
  CV_Tax = zeros(nHH,1);    
end
t2=toc/60;
CurrentTime=clock;
if DisplayFlag==1
  disp(['No Firm Response utility computation time: ',int2str(t2),' minutes.']);
  disp(['Elapsed time: ',num2str(etime(CurrentTime,StartTime)/60),' minutes.']);
end

%%  Welfare impact of tax increase: NEW NASH EQUILIBRIUM

% Compute new utility: Tax:  NEW NASH EQUILIBRIUM
if DisplayFlag==1
  disp('Computing NE utility.')
end
tic;
CV_Nash = ComputeCV_Dagsvik(V0,HHNashConsumerPrice,X,Z,basis, ...
                            EstColumn,ZIndex,iNonZero,          ...
                            xFree2,iFree2,beta2,CDiag,COffDiag, ...
                            LogNormalFlag,LogNormalIndex,       ...
                            noptions,nHH,RCDim,RCRank,nQuadAll, ...
                            Nodes,Weights);

t3=toc/60;
CurrentTime=clock;
if DisplayFlag==1
  disp(['NE utility computation time: ',int2str(t3),' minutes.']);
  disp(['Elapsed time: ',num2str(etime(CurrentTime,StartTime)/60),' minutes.']);
end
% Welfare results
%    1    HHID
%    2    CV_TAX
%    3    CV_Nash
WelfareResults = [HHID(1:noptions:noptions*(nHH-1)+1), ...
                  CV_Tax,                         ...
                  CV_Nash];
if DisplayFlag==1
  disp(['Total time: ',num2str(etime(clock,StartTime)/60),' minutes.']);              
end