clear
set more off
cap log close

***Data directories

global Path "C:/GriffithNesheimOConnell.2017.QE"

global O "$Path\MatLab_files"

global ON "$O\Nonlinear"
global OL "$O\Linear"
global OP "$O\Prefshift"
global OA "$O\Linearized"

global D "$Path\Stata_files\Results"

global DN "$D\Nonlinear"
global DL "$D\Linear"
global DP "$D\Prefshift"
global DA "$D\Linearized"

***Denotes specification to be ran
global OC "$ON"
global DC "$DN"

global C "$Path\Stata_files\Estimation"

/*
***************************************************************************
The following files contain data on the following 10 equilibria

1) Pre tax - Single product firms
2)         - Multi product firms

3) Post tax - Excise - No pricing response - Single product firms
4)                                         - Multi product firms
5)                   - Pricing response    - Single product firms
6)                                         - Multi product firms

7) Post tax - Ad valorem - No pricing response - Single product firms
8)                                             - Multi product firms
9)                       - Pricing response    - Single product firms
10)                                            - Multi product firms

The do file insheets data and saves results at
1) Market-product level
2) Product level
3) Manufacturer level
4) Household level
5) Aggregate level


A NOTE ON NAMING CONVENTION
An example name is "share_nashmultiex"

"share" refers to variable - in this case market share.

"nash" refers to equilibrium type. Can be
1) "base" for pre tax
2) "tax"  for no firm response
3) "nash" for firm response

"multi" refers to ownership structure. Can be
1) "multi"  for observed structure
2) "single" for counterfactual of all single product firms

"ex" refers to tax structure. Can be
1) "ex" for excise
2) "ad" for ad valorem

A NOTE ON PRICES
Pre tax
Consumer price = Producer price = "price_base..."

Post tax - no response
Consumer price = "price_tax..."
Producer price = "price_base..."

Post tax - response
Consumer price = "price_nash..."
Producer price = "prodprice_nash..."

***We compute compensating variation using three alternative methods
Method 1 - based on linearization of Small and Rosen 
Method 2 - based on V(p,y,x,e)=V(p',y-cv,x,e)
Method 3 - based on EV(p,y,x,e)=EV(p',y-cv,x,e)

***************************************************************************
*/

**************************
**Set up "common" data sets
**************************
insheet using "$OC/BaseTax_HHTaxResults.raw",clear

keep v1
rename v1 hh

bysort hh: keep if _n==1

sa "$DC/sampled_households.dta",replace

u "$C/pre_matlab.dta",clear

bysort hhno: keep if _n==1
keep hh hhno

merge 1:1 hhno using "$C/demweight.dta"
drop if _m==2
drop _m

merge 1:1 hhno using "$C/tripweight.dta"
drop if _m==2
drop _m

merge 1:1 hh using "$DC/sampled_households.dta"
keep if _m==3
drop _m

egen tot2 = sum(demweight)
replace demweight = demweight*(tot/tot2)
drop tot tot2

rename hhno hhno_raw
rename hh hhno

sa "$DC/hhweights.dta",replace

u "$C/pre_matlab.dta",clear

bysort prod: keep if _n==1
keep prod prodcode

sa "$C/prodcode_index.dta",replace

u "$C\combined_attributes.dta",clear

keep prodcode manu productdesc
merge 1:1 prodcode using "$C/prodcode_index.dta"
keep if _m==3
drop _m

rename prodcode mindex
rename prod prodcode

sa "$C/manufacturer.dta",replace

u "$C\nutrients.dta",clear

merge 1:1 prodcode using "$C/prodcode_index.dta"
keep if _m==3
drop _m

rename prodcode mindex
rename prod prodcode

sa "$C/intensity.dta",replace

*******************
****Insheet results
*******************

****Multi product firm results

**Excise tax

insheet using "$OC/BaseTax_HHTaxResults.raw",clear

rename v1  hhno
rename v2  manufacturer
rename v3  prodcode
rename v4  market
rename v6  cost
rename v7  volsat
rename v8  price_basemultiex
rename v9  share_basemultiex
rename v10 price_taxmultiex
rename v11 share_taxmultiex
rename v12 price_nashmultiex
rename v13 share_nashmultiex
drop v5 v14 v15

gen temp = (1/(volsat*100))*(price_taxmultiex-price_basemultiex)
su temp
local l = r(mean)
global T_ex_mu `l'
drop temp

gen choicesituation = int((_n/48)-0.01)+1

sa "$DC/Multi_excise.dta",replace

**Ad valorem tax

insheet using "$OC/AV_HHTaxResults.raw",clear

rename v1  hhno
rename v2  manufacturer
rename v3  prodcode
rename v4  market
rename v6  cost
rename v7  volsat
rename v8  price_basemulti
rename v9  share_basemulti
rename v10 price_taxmultiad
rename v11 share_taxmultiad
rename v12 price_nashmultiad
rename v13 share_nashmultiad
drop v5 v14 v15

gen temp = (1/(volsat*100))*((price_taxmultiad/price_basemulti)-1)
su temp
local l = r(mean)
global T_ad_mu `l'
drop temp

gen choicesituation = int((_n/48)-0.01)+1

sa "$DC/Multi_advalorem.dta",replace

u "$DC/Multi_advalorem.dta",clear

merge 1:1 choicesituation prodcode using "$DC/Multi_excise.dta"
tab _m
keep if _m==3
drop _m

**Check base price and share in multiproduct excise and ad valorem files are
**the same
gen checkprice = price_basemulti - price_basemultiex
gen checkshare = share_basemulti - share_basemultiex
su check*

drop price_basemultiex share_basemultiex check*

sa "$DC/Household_product_level_0.dta",replace

****Single product firm results

**Pre tax

insheet using "$OC/NoTax_HHTaxResults_Single.raw",clear

rename v1  hhno
rename v2  manufacturer
rename v3  prodcode
rename v4  market
rename v6  cost
rename v7  volsat
drop v8
drop v9
drop v10
drop v11
rename v12 price_basesingle
rename v13 share_basesingle
drop v5 v14 v15

gen choicesituation = int((_n/48)-0.01)+1

sa "$DC/Single_pretax.dta",replace

**Excise tax

insheet using "$OC/BaseTax_HHTaxResults_Single.raw",clear

rename v1  hhno
rename v2  manufacturer
rename v3  prodcode
rename v4  market
rename v6  cost
rename v7  volsat
drop v8
drop v9
rename v10 price_taxsingleex
rename v11 share_taxsingleex
rename v12 price_nashsingleex
rename v13 share_nashsingleex
drop v5 v14 v15

gen choicesituation = int((_n/48)-0.01)+1

sa "$DC/Single_excise.dta",replace

u "$DC/Single_excise.dta",clear

sort choicesituation prodcode
merge 1:1 choicesituation prodcode using "$DC/Single_pretax.dta"
tab _m
drop _m

gen temp = (1/(volsat*100))*(price_taxsingleex-price_basesingle)
su temp
local l = r(mean)
global T_ex_si `l'
drop temp

order price_basesingle share_basesingle price_taxsingleex share_taxsingleex price_nashsingleex share_nashsingleex

sa "$DC/Household_product_level_temp.dta",replace

**Ad valorem tax

insheet using "$OC/AV_HHTaxResults_Single.raw",clear

rename v1  hhno
rename v2  manufacturer
rename v3  prodcode
rename v4  market
rename v6  cost
rename v7  volsat
drop v8
drop v9
rename v10 price_taxsinglead
rename v11 share_taxsinglead
rename v12 price_nashsinglead
rename v13 share_nashsinglead
drop v5 v14 v15

gen choicesituation = int((_n/48)-0.01)+1

sa "$DC/Single_advalorem.dta",replace

u "$DC/Single_advalorem.dta",clear

merge 1:1 choicesituation prodcode using "$DC/Household_product_level_temp.dta"
tab _m
keep if _m==3
drop _m

gen temp = (1/(volsat*100))*((price_taxsinglead/price_basesingle)-1)
su temp
local l = r(mean)
global T_ad_si `l'
drop temp

sa "$DC/Household_product_level_temp.dta",replace

***Combine multi and single product firm results

u "$DC/Household_product_level_0.dta",clear

merge 1:1 choicesituation prodcode using "$DC/Household_product_level_temp.dta"
tab _m
drop _m

merge m:1 hhno using "$DC/hhweights.dta"
drop if _m==2
drop _m

replace volsat = volsat*100

gen prodprice_nashmultiex  = price_nashmultiex - $T_ex_mu*volsat
gen prodprice_nashsingleex = price_nashsingleex - $T_ex_si*volsat
gen prodprice_nashmultiad  = price_nashmultiad/(1 + $T_ad_mu*volsat)
gen prodprice_nashsinglead = price_nashsinglead/(1 + $T_ad_si*volsat)

sort choicesituation prodcode
order hhno choicesituation demweight tripweight manufacturer prodcode market cost volsat price_basemulti share_basemulti price_basesingle share_basesingle price_taxmultiex share_taxmultiex price_taxsingleex share_taxsingleex price_taxmultiad share_taxmultiad price_taxsinglead share_taxsinglead price_nashmultiex prodprice_nashmultiex share_nashmultiex price_nashsingleex prodprice_nashsingleex share_nashsingleex price_nashmultiad prodprice_nashmultiad share_nashmultiad price_nashsinglead prodprice_nashsinglead share_nashsinglead

lab var hhno            "Household code"
lab var choicesituation "Choice situation"
lab var demweight       "Demographic weight"
lab var tripweight      "Trip weight"
lab var manufacturer    "Manufacturer code"
lab var prodcode        "Product code"
lab var market          "Market code"
lab var cost            "Marginal cost"
lab var volsat          "Saturated fat (in g)"

lab var price_basemulti        "Price - pre-tax, multi"
lab var share_basemulti        "Share - pre-tax, multi"
lab var price_basesingle       "Price - pre-tax, sinlge"
lab var share_basesingle       "Share - pre-tax, single"
lab var price_taxmultiex       "Price - no response, multi, excise"
lab var share_taxmultiex       "Share - no response, multi, excise"
lab var price_taxsingleex      "Price - no response, single, excise"
lab var share_taxsingleex      "Share - no response, single, excise"
lab var price_taxmultiad       "Price - no response, multi, ad valorem"
lab var share_taxmultiad       "Share - no response, multi, ad valorem"
lab var price_taxsinglead      "Price - no response, single, ad valorem"
lab var share_taxsinglead      "Share - no response, single, ad valorem"
lab var price_nashmultiex      "Price - response, multi, excise"
lab var prodprice_nashmultiex  "Producer price - response, multi, excise"
lab var share_nashmultiex      "Share - response, multi, excise"
lab var price_nashsingleex     "Price - response, single, excise"
lab var prodprice_nashsingleex "Producer price - response, single, excise"
lab var share_nashsingleex     "Share - response, single, excise"
lab var price_nashmultiad      "Price - response, multi, ad valorem"
lab var prodprice_nashmultiad  "Producer price - response, multi, ad valorem"
lab var share_nashmultiad      "Share - response, multi, ad valorem"
lab var price_nashsinglead     "Price - response, single, ad valorem"
lab var prodprice_nashsinglead "Producer price - response, single, ad valorem"
lab var share_nashsinglead     "Share - response, single, ad valorem"

sa "$DC/Household_product_level.dta",replace

***Compensating variation results

insheet using "$OC/BaseTax_WelfareResults.raw",clear

rename v1 hhno
rename v2 CV_taxmultiex_m1
rename v3 CV_nashmultiex_m1
rename v4 CV_taxmultiex_m2
rename v5 CV_nashmultiex_m2
rename v6 CV_taxmultiex_m3
rename v7 CV_nashmultiex_m3
drop   v8

gen choicesituation = _n

sa "$DC/cv_multi_ex.dta",replace

insheet using "$OC/AV_WelfareResults.raw",clear

rename v1 hhno
rename v2 CV_taxmultiad_m1
rename v3 CV_nashmultiad_m1
rename v4 CV_taxmultiad_m2
rename v5 CV_nashmultiad_m2
rename v6 CV_taxmultiad_m3
rename v7 CV_nashmultiad_m3
drop v8

gen choicesituation = _n

sa "$DC/cv_multi_ad.dta",replace

insheet using "$OC/BaseTax_WelfareResults_Single.raw",clear

rename v1 hhno
rename v2 CV_taxsingleex_m1
rename v3 CV_nashsingleex_m1
rename v4 CV_taxsingleex_m2
rename v5 CV_nashsingleex_m2
rename v6 CV_taxsingleex_m3
rename v7 CV_nashsingleex_m3
drop v8

gen choicesituation = _n

sa "$DC/cv_single_ex.dta",replace

insheet using "$OC/AV_WelfareResults_Single.raw",clear

rename v1 hhno
rename v2 CV_taxsinglead_m1
rename v3 CV_nashsinglead_m1
rename v4 CV_taxsinglead_m2
rename v5 CV_nashsinglead_m2
rename v6 CV_taxsinglead_m3
rename v7 CV_nashsinglead_m3
drop v8

gen choicesituation = _n

sa "$DC/cv_single_ad.dta",replace

u "$DC/cv_multi_ex.dta",clear

merge 1:1 choicesituation using "$DC/cv_multi_ad.dta"
tab _m
keep if _m==3
drop _m

merge 1:1 choicesituation using "$DC/cv_single_ex.dta"
tab _m
keep if _m==3
drop _m

merge 1:1 choicesituation using "$DC/cv_single_ad.dta"
tab _m
keep if _m==3
drop _m

order hhno choicesituation CV_taxmultiex_m1 CV_taxsingleex_m1 CV_taxmultiad_m1 CV_taxsinglead_m1 CV_nashmultiex_m1 CV_nashsingleex_m1 CV_nashmultiad_m1 CV_nashsinglead_m1 CV_taxmultiex_m2 CV_taxsingleex_m2 CV_taxmultiad_m2 CV_taxsinglead_m2 CV_nashmultiex_m2 CV_nashsingleex_m2 CV_nashmultiad_m2 CV_nashsinglead_m2 CV_taxmultiex_m3 CV_taxsingleex_m3 CV_taxmultiad_m3 CV_taxsinglead_m3 CV_nashmultiex_m3 CV_nashsingleex_m3 CV_nashmultiad_m3 CV_nashsinglead_m3

lab var hhno            "Household code"
lab var choicesituation "Choice situation"

lab var CV_taxmultiex_m1   "CV - no response, multi, excise (method 1)"
lab var CV_taxsingleex_m1  "CV - no response, single, excise (method 1)"
lab var CV_taxmultiad_m1   "CV - no response, multi, ad valorem (method 1)"
lab var CV_taxsinglead_m1  "CV - no response, single, ad valorem (method 1)"
lab var CV_nashmultiex_m1  "CV - response, multi, excise (method 1)"
lab var CV_nashsingleex_m1 "CV - response, single, excise (method 1)"
lab var CV_nashmultiad_m1  "CV - response, multi, ad valorem (method 1)"
lab var CV_nashsinglead_m1 "CV - response, single, ad valorem (method 1)"
lab var CV_taxmultiex_m2   "CV - no response, multi, excise (method 2)"
lab var CV_taxsingleex_m2  "CV - no response, single, excise (method 2)"
lab var CV_taxmultiad_m2   "CV - no response, multi, ad valorem (method 2)"
lab var CV_taxsinglead_m2  "CV - no response, single, ad valorem (method 2)"
lab var CV_nashmultiex_m2  "CV - response, multi, excise (method 2)"
lab var CV_nashsingleex_m2 "CV - response, single, excise (method 2)"
lab var CV_nashmultiad_m2  "CV - response, multi, ad valorem (method 2)"
lab var CV_nashsinglead_m2 "CV - response, single, ad valorem (method 2)"
lab var CV_taxmultiex_m3   "CV - no response, multi, excise (method 3)"
lab var CV_taxsingleex_m3  "CV - no response, single, excise (method 3)"
lab var CV_taxmultiad_m3   "CV - no response, multi, ad valorem (method 3)"
lab var CV_taxsinglead_m3  "CV - no response, single, ad valorem (method 3)"
lab var CV_nashmultiex_m3  "CV - response, multi, excise (method 3)"
lab var CV_nashsingleex_m3 "CV - response, single, excise (method 3)"
lab var CV_nashmultiad_m3  "CV - response, multi, ad valorem (method 3)"
lab var CV_nashsinglead_m3 "CV - response, single, ad valorem (method 3)"

sort hhno

sa "$DC/Household_level_CV.dta",replace

************************************
****Generate results at market level
************************************

u "$DC/Household_product_level.dta",clear

sort prodcode market

egen temp1   = sum(share_basemulti*tripweight*demweight),by(prodcode)
gen  temp2   = (share_basemulti*tripweight*demweight)/temp1
egen mweight = sum(temp2),by(prodcode market)

lab var mweight "Product specific market weight"

drop temp*

egen temp1 = sum(tripweight*demweight),by(prodcode market)
gen  temp2 = (tripweight*demweight)/temp1

replace share_basemulti = share_basemulti*temp2
egen share_basemulti2 = sum(share_basemulti),by(prodcode market)

drop share_basemulti
rename share_basemulti2 share_basemulti

qui by prodcode market: gen n = _n
keep if n==1

drop manu
sort prodcode
merge m:1 prodcode using "$C/manufacturer.dta"
tab _m
keep if _m==3 | prodcode==1
drop _m

keep  prodcode productdesc manufacturer market price_basemulti price_basesingle price_taxmultiex price_taxsingleex price_taxmultiad price_taxsinglead price_nashmultiex prodprice_nashmultiex price_nashsingleex prodprice_nashsingleex price_nashmultiad prodprice_nashmultiad price_nashsinglead prodprice_nashsinglead cost mweight share_basemulti
order prodcode productdesc manufacturer market price_basemulti price_basesingle price_taxmultiex price_taxsingleex price_taxmultiad price_taxsinglead price_nashmultiex prodprice_nashmultiex price_nashsingleex prodprice_nashsingleex price_nashmultiad prodprice_nashmultiad price_nashsinglead prodprice_nashsinglead cost mweight share_basemulti

sa "$DC/Market_results.dta",replace

******************************************
****Generate results at product level
******************************************
u "$DC/Household_product_level.dta",clear

replace manufacturer = manufacturer - 1
rename manufacturer man

sort prodcode
merge m:1 prodcode using "$C/manufacturer.dta"
tab _m
keep if _m==3 | prodcode==1
drop _m

sort prodcode
merge m:1 prodcode using "$C/intensity.dta"
tab _m
keep if _m==3 | prodcode==1
drop _m

**Generate profit for each equilibrium, weighted with sampling weight

gen prof_basemulti    = share_basemulti* (price_basemulti - cost) *tripweight*demweight
gen prof_basesingle   = share_basesingle*(price_basesingle - cost)*tripweight*demweight

gen prof_taxmultiex   = share_taxmultiex* (price_basemulti - cost) *tripweight*demweight
gen prof_taxsingleex  = share_taxsingleex*(price_basesingle - cost)*tripweight*demweight

gen prof_taxmultiad   = share_taxmultiad* (price_basemulti - cost)*tripweight*demweight
gen prof_taxsinglead  = share_taxsinglead*(price_basesingle - cost)*tripweight*demweight

gen prof_nashmultiex  = share_nashmultiex* (prodprice_nashmultiex - cost) *tripweight*demweight
gen prof_nashsingleex = share_nashsingleex*(prodprice_nashsingleex - cost)*tripweight*demweight

gen prof_nashmultiad  = share_nashmultiad* (prodprice_nashmultiad - cost) *tripweight*demweight
gen prof_nashsinglead = share_nashsinglead*(prodprice_nashsinglead - cost)*tripweight*demweight

lab var prof_basemulti    "Profit - pre-tax, multi"
lab var prof_basesingle   "Profit - pre-tax, single"
lab var prof_taxmultiex   "Profit - no response, multi, excise"
lab var prof_taxsingleex  "Profit - no response, single, excise"
lab var prof_taxmultiad   "Profit - no response, multi, ad valorem"
lab var prof_taxsinglead  "Profit - no response, single, ad valorem"
lab var prof_nashmultiex  "Profit - response, multi, excise"
lab var prof_nashsingleex "Profit - response, single, excise"
lab var prof_nashmultiad  "Profit - response, multi, ad valorem"
lab var prof_nashsinglead "Profit - response, single, ad valorem"

**Weight price for each equilibrium by share and sample weights

egen temp1    = sum(share_basemulti*tripweight*demweight),by(prodcode)
gen  temp2_bm = share_basemulti*tripweight*demweight/temp1
drop temp1
egen temp1    = sum(share_basesingle*tripweight*demweight),by(prodcode)
gen  temp2_bs = share_basesingle*tripweight*demweight/temp1
drop temp1

egen temp1     = sum(share_taxmultiex*tripweight*demweight),by(prodcode)
gen  temp2_tme = share_taxmultiex*tripweight*demweight/temp1
drop temp1
egen temp1     = sum(share_taxsingleex*tripweight*demweight),by(prodcode)
gen  temp2_tse = share_taxsingleex*tripweight*demweight/temp1
drop temp1

egen temp1     = sum(share_taxmultiad*tripweight*demweight),by(prodcode)
gen  temp2_tma = share_taxmultiad*tripweight*demweight/temp1
drop temp1
egen temp1     = sum(share_taxsinglead*tripweight*demweight),by(prodcode)
gen  temp2_tsa = share_taxsinglead*tripweight*demweight/temp1
drop temp1

egen temp1     = sum(share_nashmultiex*tripweight*demweight),by(prodcode)
gen  temp2_nme = share_nashmultiex*tripweight*demweight/temp1
drop temp1
egen temp1     = sum(share_nashsingleex*tripweight*demweight),by(prodcode)
gen  temp2_nse = share_nashsingleex*tripweight*demweight/temp1
drop temp1

egen temp1     = sum(share_nashmultiad*tripweight*demweight),by(prodcode)
gen  temp2_nma = share_nashmultiad*tripweight*demweight/temp1
drop temp1
egen temp1     = sum(share_nashsinglead*tripweight*demweight),by(prodcode)
gen  temp2_nsa = share_nashsinglead*tripweight*demweight/temp1
drop temp1

**Note average cost is calculate at the base multi product firm equilibrium
replace cost = temp2_bm*cost

replace price_basemulti        = temp2_bm* price_basemulti
replace price_basesingle       = temp2_bs* price_basesingle

replace price_taxmultiex       = temp2_tme*price_taxmultiex
replace price_taxsingleex      = temp2_tse*price_taxsingleex

replace price_taxmultiad       = temp2_tma*price_taxmultiad
replace price_taxsinglead      = temp2_tsa*price_taxsinglead

replace price_nashmultiex      = temp2_nme*price_nashmultiex
replace price_nashsingleex     = temp2_nse*price_nashsingleex

replace prodprice_nashmultiex  = temp2_nme*prodprice_nashmultiex
replace prodprice_nashsingleex = temp2_nse*prodprice_nashsingleex

replace price_nashmultiad      = temp2_nma*price_nashmultiad
replace price_nashsinglead     = temp2_nsa*price_nashsinglead

replace prodprice_nashmultiad  = temp2_nma*prodprice_nashmultiad
replace prodprice_nashsinglead = temp2_nsa*prodprice_nashsinglead

drop temp*

**Weight share for each equilibrium by sample weights

egen temp1 = sum(tripweight*demweight),by(prodcode)
gen  temp2 = (tripweight*demweight)/temp1

replace share_basemulti    = temp2*share_basemulti
replace share_basesingle   = temp2*share_basesingle

replace share_taxmultiex   = temp2*share_taxmultiex
replace share_taxsingleex  = temp2*share_taxsingleex

replace share_taxmultiad   = temp2*share_taxmultiad
replace share_taxsinglead  = temp2*share_taxsinglead

replace share_nashmultiex  = temp2*share_nashmultiex
replace share_nashsingleex = temp2*share_nashsingleex

replace share_nashmultiad  = temp2*share_nashmultiad
replace share_nashsinglead = temp2*share_nashsinglead

collapse (mean) volsat saturates (sum) cost price_basemulti price_basesingle price_taxmultiex price_taxsingleex price_taxmultiad price_taxsinglead price_nashmultiex prodprice_nashmultiex price_nashsingleex prodprice_nashsingleex price_nashmultiad prodprice_nashmultiad price_nashsinglead prodprice_nashsinglead prof_basemulti prof_basesingle prof_taxmultiex prof_taxsingleex prof_taxmultiad prof_taxsinglead prof_nashmultiex prof_nashsingleex prof_nashmultiad prof_nashsinglead share_basemulti share_basesingle share_taxmultiex share_taxsingleex share_taxmultiad share_taxsinglead share_nashmultiex share_nashsingleex share_nashmultiad share_nashsinglead,by(manufacturer man prodcode productdesc)

sa "$DC/Product_results_temp.dta",replace

u "$DC/Product_results_temp.dta",clear

lab var saturates "Saturated fat per 100g"
lab var cost      "Mean marginal cost"

***Pass through variables

gen pass_multiex  = (price_nashmultiex -  price_basemulti) /(price_taxmultiex -  price_basemulti)
gen pass_singleex = (price_nashsingleex - price_basesingle)/(price_taxsingleex - price_basesingle)
gen pass_multiad  = (price_nashmultiad -  price_basemulti) /(price_taxmultiad -  price_basemulti)
gen pass_singlead = (price_nashsinglead - price_basesingle)/(price_taxsinglead - price_basesingle)
lab var pass_multiex  "Pass through - multi, excise"
lab var pass_singleex "Pass through - single, excise"
lab var pass_multiad  "Pass through - multi, ad valorem"
lab var pass_singlead "Pass through - single, ad valorem"

***Margins

gen margin_basemulti    = (price_basemulti-cost)   /price_basemulti
gen margin_basesingle   = (price_basesingle-cost)  /price_basesingle

gen margin_taxmultiex   = (price_taxmultiex-cost)  /price_taxmultiex
gen margin_taxsingleex  = (price_taxsingleex-cost) /price_taxsingleex

gen margin_taxmultiad   = (price_taxmultiad-cost)  /price_taxmultiad
gen margin_taxsinglead  = (price_taxsinglead-cost) /price_taxsinglead

gen margin_nashmultiex  = (price_nashmultiex-cost) /price_nashmultiex
gen margin_nashsingleex = (price_nashsingleex-cost)/price_nashsingleex

gen margin_nashmultiad  = (price_nashmultiad-cost) /price_nashmultiad
gen margin_nashsinglead = (price_nashsinglead-cost)/price_nashsinglead

lab var margin_basemulti    "Average margin - pre-tax, multi"
lab var margin_basesingle   "Average margin - pre-tax, single"

lab var margin_taxmultiex   "Average margin - no response, multi, excise"
lab var margin_taxsingleex  "Average margin - no response, single, excise"

lab var margin_taxmultiad   "Average margin - no response, multi, ad valorem"
lab var margin_taxsinglead  "Average margin - no response, single, ad valorem"

lab var margin_nashmultiex  "Average margin - response, multi, excise"
lab var margin_nashsingleex "Average margin - response, single, excise"

lab var margin_nashmultiad  "Average margin - response, multi, ad valorem"
lab var margin_nashsinglead "Average margin - response, single, ad valorem"

***Rescaled shares
gen size = (vols/sat)/10

foreach var in multi single {
    gen temp = share_base`var' if prodcode!=1
    replace temp = temp*size
    egen temp2 = sum(temp)
    gen reshare_base`var' = temp/temp2
    drop temp temp2
}


foreach var1 in tax nash {
    foreach var2 in multi single {
        foreach var3 in ex ad {
            gen temp = share_`var1'`var2'`var3' if prodcode!=1
            replace temp = temp*size
            egen temp2 = sum(temp)
            gen reshare_`var1'`var2'`var3' = temp/temp2
            drop temp temp2
        }
    }
}

drop size

lab var reshare_basemulti    "Quantity share - pre-tax, multi"
lab var reshare_basesingle   "Quantity share - pre-tax, single"
lab var reshare_taxmultiex   "Quantity share - no response, multi, excise"
lab var reshare_taxsingleex  "Quantity share - no response, single, excise"
lab var reshare_taxmultiad   "Quantity share - no response, multi, ad valorem"
lab var reshare_taxsinglead  "Quantity share - no response, single, ad valorem"
lab var reshare_nashmultiex  "Quantity share - response, multi, excise"
lab var reshare_nashsingleex "Quantity share - response, single, excise"
lab var reshare_nashmultiad  "Quantity share - response, multi, ad valorem"
lab var reshare_nashsinglead "Quantity share - response, single, ad valorem"

sort prodcode

sa "$DC/Product_results.dta",replace

******************************************
****Generate results at manufacturer level
******************************************

u "$DC/Product_results.dta",clear

drop if prodcode==1

gen x = -share_basemulti
sort manufacturer x
by manufacturer x: gen n = _n
keep if n==1
keep manufacturer productdesc

sort manufacturer
by manufacturer: keep if _n==1
sort manufacturer

sa "$DC/biggest_product.dta",replace

u "$DC/Product_results.dta",clear

egen temp1 = sum(share_basemulti),by(manufacturer)
gen  temp2 = share_basemulti/temp1

foreach var in price_basemulti volsat cost margin_basemulti {
    replace `var'=`var'*temp2
}
drop temp1 temp2

egen temp1 = sum(share_basesingle),by(manufacturer)
gen  temp2 = share_basesingle/temp1
replace price_basesingle  = price_basesingle*temp2
replace margin_basesingle = margin_basesingle*temp2
drop temp1 temp2

foreach var1 in tax nash {
    foreach var2 in multi single {
        foreach var3 in ex ad {
            egen temp1 = sum(share_`var1'`var2'`var3'),by(manufacturer)
            gen  temp2 = share_`var1'`var2'`var3'/temp1
            replace price_`var1'`var2'`var3' = price_`var1'`var2'`var3'*temp2
            drop temp1 temp2
        }
    }
}

foreach var2 in multi single {
    foreach var3 in ex ad {
        egen temp1 = sum(share_nash`var2'`var3'),by(manufacturer)
        gen  temp2 = share_nash`var2'`var3'/temp1
        replace prodprice_nash`var2'`var3' = prodprice_nash`var2'`var3'*temp2
        drop temp1 temp2
    }
}

gen n = 1

collapse (sum) n share_basemulti share_basesingle share_taxmultiex share_taxsingleex share_taxmultiad share_taxsinglead share_nashmultiex share_nashsingleex share_nashmultiad share_nashsinglead reshare_basemulti reshare_basesingle reshare_taxmultiex reshare_taxsingleex reshare_taxmultiad reshare_taxsinglead reshare_nashmultiex reshare_nashsingleex reshare_nashmultiad reshare_nashsinglead price_basemulti price_basesingle price_taxmultiex price_taxsingleex price_taxmultiad price_taxsinglead price_nashmultiex prodprice_nashmultiex price_nashsingleex prodprice_nashsingleex price_nashmultiad prodprice_nashmultiad price_nashsinglead prodprice_nashsinglead volsat cost margin_basemulti margin_basesingle prof_basemulti prof_basesingle prof_taxmultiex prof_taxsingleex prof_taxmultiad prof_taxsinglead prof_nashmultiex prof_nashsingleex prof_nashmultiad prof_nashsinglead, by(manufacturer man)

lab var n "Number of products"

sort manufacturer
merge 1:1 manufacturer using "$DC/biggest_product.dta"
tab _m
drop _m

rename productdesc largestprod
lab var largestprod "Product with largest market share"

gen pass_multiex  = (price_nashmultiex -  price_basemulti) /(price_taxmultiex  - price_basemulti)
gen pass_singleex = (price_nashsingleex - price_basesingle)/(price_taxsingleex - price_basesingle)
gen pass_multiad  = (price_nashmultiad -  price_basemulti) /(price_taxmultiad  - price_basemulti)
gen pass_singlead = (price_nashsinglead - price_basesingle)/(price_taxsinglead - price_basesingle)
lab var pass_multiex  "Pass through - multi, excise"
lab var pass_singleex "Pass through - single, excise"
lab var pass_multiad  "Pass through - multi, ad valorem"
lab var pass_singlead "Pass through - single, ad valorem"

foreach var in prof_basemulti prof_basesingle prof_taxmultiex prof_taxsingleex prof_taxmultiad prof_taxsinglead prof_nashmultiex prof_nashsingleex prof_nashmultiad prof_nashsinglead {
    replace `var' = `var'/1000000
}

sort manufacturer

sa "$DC/Manufacturer_results.dta",replace

**********************************************
****Generate results at choice situation level
**********************************************

u "$DC/Household_product_level.dta",clear

sort prodcode
merge m:1 prodcode using "$C/intensity.dta"
tab _m
keep if _m==3 | prodcode==1
drop _m

drop manufacturer market mindex saturates sodium

**Nutrients

gen sat_basemulti    = share_basemulti   *volsat
gen sat_basesingle   = share_basesingle  *volsat
gen sat_taxmultiex   = share_taxmultiex  *volsat
gen sat_taxsingleex  = share_taxsingleex *volsat
gen sat_taxmultiad   = share_taxmultiad  *volsat
gen sat_taxsinglead  = share_taxsinglead *volsat
gen sat_nashmultiex  = share_nashmultiex *volsat
gen sat_nashsingleex = share_nashsingleex*volsat
gen sat_nashmultiad  = share_nashmultiad *volsat
gen sat_nashsinglead = share_nashsinglead*volsat

lab var sat_basemulti    "Saturated fat - pre-tax, multi"
lab var sat_basesingle   "Saturated fat - pre-tax, single"
lab var sat_taxmultiex   "Saturated fat - no response, multi, excise"
lab var sat_taxsingleex  "Saturated fat - no response, single, excise"
lab var sat_taxmultiad   "Saturated fat - no response, multi, ad valorem"
lab var sat_taxsinglead  "Saturated fat - no response, single, ad valorem"
lab var sat_nashmultiex  "Saturated fat - response, multi, excise"
lab var sat_nashsingleex "Saturated fat - response, single, excise"
lab var sat_nashmultiad  "Saturated fat - response, multi, ad valorem"
lab var sat_nashsinglead "Saturated fat - response, single, ad valorem"

**Outside

gen x_bm  = share_basemulti    if prodcode==1
gen x_bs  = share_basesingle   if prodcode==1
gen x_tme = share_taxmultiex   if prodcode==1
gen x_tse = share_taxsingleex  if prodcode==1
gen x_tma = share_taxmultiad   if prodcode==1
gen x_tsa = share_taxsinglead  if prodcode==1
gen x_nme = share_nashmultiex  if prodcode==1
gen x_nse = share_nashsingleex if prodcode==1
gen x_nma = share_nashmultiad  if prodcode==1
gen x_nsa = share_nashsinglead if prodcode==1

egen out_basemulti    = min(x_bm ),by(hhno)
egen out_basesingle   = min(x_bs ),by(hhno)
egen out_taxmultiex   = min(x_tme),by(hhno)
egen out_taxsingleex  = min(x_tse),by(hhno)
egen out_taxmultiad   = min(x_tma),by(hhno)
egen out_taxsinglead  = min(x_tsa),by(hhno)
egen out_nashmultiex  = min(x_nme),by(hhno)
egen out_nashsingleex = min(x_nse),by(hhno)
egen out_nashmultiad  = min(x_nma),by(hhno)
egen out_nashsinglead = min(x_nsa),by(hhno)

drop x_*

lab var out_basemulti    "Outside option - pre-tax, multi"
lab var out_basesingle   "Outside option - pre-tax, single"
lab var out_taxmultiex   "Outside option - no response, multi, excise"
lab var out_taxsingleex  "Outside option - no response, single, excise"
lab var out_taxmultiad   "Outside option - no response, multi, ad valorem"
lab var out_taxsinglead  "Outside option - no response, single, ad valorem"
lab var out_nashmultiex  "Outside option - response, multi, excise"
lab var out_nashsingleex "Outside option - response, single, excise"
lab var out_nashmultiad  "Outside option - response, multi, ad valorem"
lab var out_nashsinglead "Outside option - response, single, ad valorem"

**Expenditure, costs, profits and tax

gen exp_basemulti    = share_basemulti   *price_basemulti
gen exp_basesingle   = share_basesingle  *price_basesingle
gen exp_taxmultiex   = share_taxmultiex  *price_taxmultiex
gen exp_taxsingleex  = share_taxsingleex *price_taxsingleex
gen exp_taxmultiad   = share_taxmultiad  *price_taxmultiad
gen exp_taxsinglead  = share_taxsinglead *price_taxsinglead
gen exp_nashmultiex  = share_nashmultiex *price_nashmultiex
gen exp_nashsingleex = share_nashsingleex*price_nashsingleex
gen exp_nashmultiad  = share_nashmultiad *price_nashmultiad
gen exp_nashsinglead = share_nashsinglead*price_nashsinglead

lab var exp_basemulti    "Expenditure - pre-tax, multi"
lab var exp_basesingle   "Expenditure - pre-tax, single"
lab var exp_taxmultiex   "Expenditure - no response, multi, excise"
lab var exp_taxsingleex  "Expenditure - no response, single, excise"
lab var exp_taxmultiad   "Expenditure - no response, multi, ad valorem"
lab var exp_taxsinglead  "Expenditure - no response, single, ad valorem"
lab var exp_nashmultiex  "Expenditure - response, multi, excise"
lab var exp_nashsingleex "Expenditure - response, single, excise"
lab var exp_nashmultiad  "Expenditure - response, multi, ad valorem"
lab var exp_nashsinglead "Expenditure - response, single, ad valorem"

gen cost_basemulti    = share_basemulti   *cost
gen cost_basesingle   = share_basesingle  *cost
gen cost_taxmultiex   = share_taxmultiex  *cost
gen cost_taxsingleex  = share_taxsingleex *cost
gen cost_taxmultiad   = share_taxmultiad  *cost
gen cost_taxsinglead  = share_taxsinglead *cost
gen cost_nashmultiex  = share_nashmultiex *cost
gen cost_nashsingleex = share_nashsingleex*cost
gen cost_nashmultiad  = share_nashmultiad *cost
gen cost_nashsinglead = share_nashsinglead*cost

lab var cost_basemulti    "Cost - pre-tax, multi"
lab var cost_basesingle   "Cost - pre-tax, single"
lab var cost_taxmultiex   "Cost - no response, multi, excise"
lab var cost_taxsingleex  "Cost - no response, single, excise"
lab var cost_taxmultiad   "Cost - no response, multi, ad valorem"
lab var cost_taxsinglead  "Cost - no response, single, ad valorem"
lab var cost_nashmultiex  "Cost - response, multi, excise"
lab var cost_nashsingleex "Cost - response, single, excise"
lab var cost_nashmultiad  "Cost - response, multi, ad valorem"
lab var cost_nashsinglead "Cost - response, single, ad valorem"

gen prof_basemulti    = share_basemulti   *(price_basemulti -        cost)
gen prof_basesingle   = share_basesingle  *(price_basesingle -       cost)
gen prof_taxmultiex   = share_taxmultiex  *(price_basemulti -        cost)
gen prof_taxsingleex  = share_taxsingleex *(price_basesingle -       cost)
gen prof_taxmultiad   = share_taxmultiad  *(price_basemulti -        cost)
gen prof_taxsinglead  = share_taxsinglead *(price_basesingle -       cost)
gen prof_nashmultiex  = share_nashmultiex *(prodprice_nashmultiex -  cost)
gen prof_nashsingleex = share_nashsingleex*(prodprice_nashsingleex - cost)
gen prof_nashmultiad  = share_nashmultiad *(prodprice_nashmultiad -  cost)
gen prof_nashsinglead = share_nashsinglead*(prodprice_nashsinglead - cost)

lab var prof_basemulti    "Profit - pre-tax, multi"
lab var prof_basesingle   "Profit - pre-tax, single"
lab var prof_taxmultiex   "Profit - no response, multi, excise"
lab var prof_taxsingleex  "Profit - no response, single, excise"
lab var prof_taxmultiad   "Profit - no response, multi, ad valorem"
lab var prof_taxsinglead  "Profit - no response, single, ad valorem"
lab var prof_nashmultiex  "Profit - response, multi, excise"
lab var prof_nashsingleex "Profit - response, single, excise"
lab var prof_nashmultiad  "Profit - response, multi, ad valorem"
lab var prof_nashsinglead "Profit - response, single, ad valorem"

drop cost price_taxmultiex price_taxsingleex price_taxmultiad price_taxsinglead price_nashmultiex price_nashsingleex price_nashmultiad prodprice_nashsingleex prodprice_nashsingleex price_nashsinglead

gen tax_taxmultiex   = share_taxmultiex  *$T_ex_mu*volsat
gen tax_taxsingleex  = share_taxsingleex *$T_ex_si*volsat
gen tax_taxmultiad   = share_taxmultiad  *$T_ad_mu*volsat*price_basemulti
gen tax_taxsinglead  = share_taxsinglead *$T_ad_si*volsat*price_basesingle
gen tax_nashmultiex  = share_nashmultiex *$T_ex_mu*volsat
gen tax_nashsingleex = share_nashsingleex*$T_ex_si*volsat
gen tax_nashmultiad  = share_nashmultiad *$T_ad_mu*volsat*prodprice_nashmultiad
gen tax_nashsinglead = share_nashsinglead*$T_ad_si*volsat*prodprice_nashsinglead

lab var tax_taxmultiex   "Tax - no response, multi, excise"
lab var tax_taxsingleex  "Tax - no response, single, excise"
lab var tax_taxmultiad   "Tax - no response, multi, ad valorem"
lab var tax_taxsinglead  "Tax - no response, single, ad valorem"
lab var tax_nashmultiex  "Tax - response, multi, excise"
lab var tax_nashsingleex "Tax - response, single, excise"
lab var tax_nashmultiad  "Tax - response, multi, ad valorem"
lab var tax_nashsinglead "Tax - response, single, ad valorem"

keep hhno choicesituation demweight tripweight out_basemulti out_basesingle out_taxmultiex out_taxsingleex out_taxmultiad out_taxsinglead out_nashmultiex out_nashsingleex out_nashmultiad out_nashsinglead sat_basemulti sat_basesingle sat_taxmultiex sat_taxsingleex sat_taxmultiad sat_taxsinglead sat_nashmultiex sat_nashsingleex sat_nashmultiad sat_nashsinglead exp_basemulti exp_basesingle exp_taxmultiex exp_taxsingleex exp_taxmultiad exp_taxsinglead exp_nashmultiex exp_nashsingleex exp_nashmultiad exp_nashsinglead cost_basemulti cost_basesingle cost_taxmultiex cost_taxsingleex cost_taxmultiad cost_taxsinglead cost_nashmultiex cost_nashsingleex cost_nashmultiad cost_nashsinglead prof_basemulti prof_basesingle prof_taxmultiex prof_taxsingleex prof_taxmultiad prof_taxsinglead prof_nashmultiex prof_nashsingleex prof_nashmultiad prof_nashsinglead tax_taxmultiex tax_taxsingleex tax_taxmultiad  tax_taxsinglead tax_nashmultiex tax_nashsingleex tax_nashmultiad tax_nashsinglead

collapse (mean) demweight tripweight out_basemulti out_basesingle out_taxmultiex out_taxsingleex out_taxmultiad out_taxsinglead out_nashmultiex out_nashsingleex out_nashmultiad out_nashsinglead (sum) sat_basemulti sat_basesingle sat_taxmultiex sat_taxsingleex sat_taxmultiad sat_taxsinglead sat_nashmultiex sat_nashsingleex sat_nashmultiad sat_nashsinglead exp_basemulti exp_basesingle exp_taxmultiex exp_taxsingleex exp_taxmultiad exp_taxsinglead exp_nashmultiex exp_nashsingleex exp_nashmultiad exp_nashsinglead cost_basemulti cost_basesingle cost_taxmultiex cost_taxsingleex cost_taxmultiad cost_taxsinglead cost_nashmultiex cost_nashsingleex cost_nashmultiad cost_nashsinglead prof_basemulti prof_basesingle prof_taxmultiex prof_taxsingleex prof_taxmultiad prof_taxsinglead prof_nashmultiex prof_nashsingleex prof_nashmultiad prof_nashsinglead tax_taxmultiex tax_taxsingleex tax_taxmultiad  tax_taxsinglead tax_nashmultiex tax_nashsingleex tax_nashmultiad tax_nashsinglead,by(hhno choicesituation)

foreach var in sat_basemulti sat_basesingle sat_taxmultiex sat_taxsingleex sat_taxmultiad sat_taxsinglead sat_nashmultiex sat_nashsingleex sat_nashmultiad sat_nashsinglead  {
    replace `var' = `var'/1000
}

merge 1:1 choicesituation using "$DC/Household_level_CV.dta"
tab _m
drop _m

**Consumer cost of a 1Kg reduction in saturated fat

foreach n of numlist 1 2 3 {

gen cfatcost_taxmultiex_m`n'   = -CV_taxmultiex_m`n'/  (sat_taxmultiex   - sat_basemulti )
gen cfatcost_taxsingleex_m`n'  = -CV_taxsingleex_m`n'/ (sat_taxsingleex  - sat_basesingle)
gen cfatcost_taxmultiad_m`n'   = -CV_taxmultiad_m`n'/  (sat_taxmultiad   - sat_basemulti )
gen cfatcost_taxsinglead_m`n'  = -CV_taxsinglead_m`n'/ (sat_taxsinglead  - sat_basesingle)
gen cfatcost_nashmultiex_m`n'  = -CV_nashmultiex_m`n'/ (sat_nashmultiex  - sat_basemulti )
gen cfatcost_nashsingleex_m`n' = -CV_nashsingleex_m`n'/(sat_nashsingleex - sat_basesingle)
gen cfatcost_nashmultiad_m`n'  = -CV_nashmultiad_m`n'/ (sat_nashmultiad  - sat_basemulti )
gen cfatcost_nashsinglead_m`n' = -CV_nashsinglead_m`n'/(sat_nashsinglead - sat_basesingle)

lab var cfatcost_taxmultiex_m`n'   "Consumer cost of 1kg fat reduction - no response, multi, excise (Method `n')"
lab var cfatcost_taxsingleex_m`n'  "Consumer cost of 1kg fat reduction - no response, single, excise (Method `n')"
lab var cfatcost_taxmultiad_m`n'   "Consumer cost of 1kg fat reduction - no response, multi, ad valorem (Method `n')"
lab var cfatcost_taxsinglead_m`n'  "Consumer cost of 1kg fat reduction - no response, single, ad valorem (Method `n')"
lab var cfatcost_nashmultiex_m`n'  "Consumer cost of 1kg fat reduction - response, multi, excise (Method `n')"
lab var cfatcost_nashsingleex_m`n' "Consumer cost of 1kg fat reduction - response, single, excise (Method `n')"
lab var cfatcost_nashmultiad_m`n'  "Consumer cost of 1kg fat reduction - response, multi, ad valorem (Method `n')"
lab var cfatcost_nashsinglead_m`n' "Consumer cost of 1kg fat reduction - response, single, ad valorem (Method `n')"

}

foreach var in sat_basemulti sat_basesingle sat_taxmultiex sat_taxsingleex sat_taxmultiad sat_taxsinglead sat_nashmultiex sat_nashsingleex sat_nashmultiad sat_nashsinglead exp_basemulti exp_basesingle exp_taxmultiex exp_taxsingleex exp_taxmultiad exp_taxsinglead exp_nashmultiex exp_nashsingleex exp_nashmultiad exp_nashsinglead cost_basemulti cost_basesingle cost_taxmultiex cost_taxsingleex cost_taxmultiad cost_taxsinglead cost_nashmultiex cost_nashsingleex cost_nashmultiad cost_nashsinglead prof_basemulti prof_basesingle prof_taxmultiex prof_taxsingleex prof_taxmultiad prof_taxsinglead prof_nashmultiex prof_nashsingleex prof_nashmultiad prof_nashsinglead tax_taxmultiex tax_taxsingleex tax_taxmultiad  tax_taxsinglead tax_nashmultiex tax_nashsingleex tax_nashmultiad tax_nashsinglead CV_taxmultiex_m1 CV_taxsingleex_m1 CV_taxmultiad_m1 CV_taxsinglead_m1 CV_nashmultiex_m1 CV_nashsingleex_m1 CV_nashmultiad_m1 CV_nashsinglead_m1 CV_taxmultiex_m2 CV_taxsingleex_m2 CV_taxmultiad_m2 CV_taxsinglead_m2 CV_nashmultiex_m2 CV_nashsingleex_m2 CV_nashmultiad_m2 CV_nashsinglead_m2 CV_taxmultiex_m3 CV_taxsingleex_m3 CV_taxmultiad_m3 CV_taxsinglead_m3 CV_nashmultiex_m3 CV_nashsingleex_m3 CV_nashmultiad_m3 CV_nashsinglead_m3 {
    qui replace `var' = `var'*tripweight
}

sa "$DC/Household_results.dta",replace

***************************************
****Generate results at aggregate level
***************************************

u "$DC/Household_results.dta",clear

foreach var in sat_basemulti sat_basesingle sat_taxmultiex sat_taxsingleex sat_taxmultiad sat_taxsinglead sat_nashmultiex sat_nashsingleex sat_nashmultiad sat_nashsinglead exp_basemulti exp_basesingle exp_taxmultiex exp_taxsingleex exp_taxmultiad exp_taxsinglead exp_nashmultiex exp_nashsingleex exp_nashmultiad exp_nashsinglead cost_basemulti cost_basesingle cost_taxmultiex cost_taxsingleex cost_taxmultiad cost_taxsinglead cost_nashmultiex cost_nashsingleex cost_nashmultiad cost_nashsinglead prof_basemulti prof_basesingle prof_taxmultiex prof_taxsingleex prof_taxmultiad prof_taxsinglead prof_nashmultiex prof_nashsingleex prof_nashmultiad prof_nashsinglead tax_taxmultiex tax_taxsingleex tax_taxmultiad  tax_taxsinglead tax_nashmultiex tax_nashsingleex tax_nashmultiad tax_nashsinglead CV_taxmultiex_m1 CV_taxsingleex_m1 CV_taxmultiad_m1 CV_taxsinglead_m1 CV_nashmultiex_m1 CV_nashsingleex_m1 CV_nashmultiad_m1 CV_nashsinglead_m1 CV_taxmultiex_m2 CV_taxsingleex_m2 CV_taxmultiad_m2 CV_taxsinglead_m2 CV_nashmultiex_m2 CV_nashsingleex_m2 CV_nashmultiad_m2 CV_nashsinglead_m2 CV_taxmultiex_m3 CV_taxsingleex_m3 CV_taxmultiad_m3 CV_taxsinglead_m3 CV_nashmultiex_m3 CV_nashsingleex_m3 CV_nashmultiad_m3 CV_nashsinglead_m3 {
    replace `var' = `var'*demweight
}

collapse (sum) sat_basemulti sat_basesingle sat_taxmultiex sat_taxsingleex sat_taxmultiad sat_taxsinglead sat_nashmultiex sat_nashsingleex sat_nashmultiad sat_nashsinglead exp_basemulti exp_basesingle exp_taxmultiex exp_taxsingleex exp_taxmultiad exp_taxsinglead exp_nashmultiex exp_nashsingleex exp_nashmultiad exp_nashsinglead cost_basemulti cost_basesingle cost_taxmultiex cost_taxsingleex cost_taxmultiad cost_taxsinglead cost_nashmultiex cost_nashsingleex cost_nashmultiad cost_nashsinglead prof_basemulti prof_basesingle prof_taxmultiex prof_taxsingleex prof_taxmultiad prof_taxsinglead prof_nashmultiex prof_nashsingleex prof_nashmultiad prof_nashsinglead tax_taxmultiex tax_taxsingleex tax_taxmultiad  tax_taxsinglead tax_nashmultiex tax_nashsingleex tax_nashmultiad tax_nashsinglead CV_taxmultiex_m1 CV_taxsingleex_m1 CV_taxmultiad_m1 CV_taxsinglead_m1 CV_nashmultiex_m1 CV_nashsingleex_m1 CV_nashmultiad_m1 CV_nashsinglead_m1 CV_taxmultiex_m2 CV_taxsingleex_m2 CV_taxmultiad_m2 CV_taxsinglead_m2 CV_nashmultiex_m2 CV_nashsingleex_m2 CV_nashmultiad_m2 CV_nashsinglead_m2 CV_taxmultiex_m3 CV_taxsingleex_m3 CV_taxmultiad_m3 CV_taxsinglead_m3 CV_nashmultiex_m3 CV_nashsingleex_m3 CV_nashmultiad_m3 CV_nashsinglead_m3

foreach n of numlist 1 2 3 {

gen wel_taxmultiex_m`n'   = tax_taxmultiex   - CV_taxmultiex_m`n'   + (prof_taxmultiex   - prof_basemulti )
gen wel_taxsingleex_m`n'  = tax_taxsingleex  - CV_taxsingleex_m`n'  + (prof_taxsingleex  - prof_basesingle)
gen wel_taxmultiad_m`n'   = tax_taxmultiad   - CV_taxmultiad_m`n'   + (prof_taxmultiad   - prof_basemulti )
gen wel_taxsinglead_m`n'  = tax_taxsinglead  - CV_taxsinglead_m`n'  + (prof_taxsinglead  - prof_basesingle)
gen wel_nashmultiex_m`n'  = tax_nashmultiex  - CV_nashmultiex_m`n'  + (prof_nashmultiex  - prof_basemulti )
gen wel_nashsingleex_m`n' = tax_nashsingleex - CV_nashsingleex_m`n' + (prof_nashsingleex - prof_basesingle)
gen wel_nashmultiad_m`n'  = tax_nashmultiad  - CV_nashmultiad_m`n'  + (prof_nashmultiad  - prof_basemulti )
gen wel_nashsinglead_m`n' = tax_nashsinglead - CV_nashsinglead_m`n' + (prof_nashsinglead - prof_basesingle)

gen tfatcost_taxmultiex_m`n'   = wel_taxmultiex_m`n'/  (sat_taxmultiex   - sat_basemulti )
gen tfatcost_taxsingleex_m`n'  = wel_taxsingleex_m`n'/ (sat_taxsingleex  - sat_basesingle)
gen tfatcost_taxmultiad_m`n'   = wel_taxmultiad_m`n'/  (sat_taxmultiad   - sat_basemulti )
gen tfatcost_taxsinglead_m`n'  = wel_taxsinglead_m`n'/ (sat_taxsinglead  - sat_basesingle)
gen tfatcost_nashmultiex_m`n'  = wel_nashmultiex_m`n'/ (sat_nashmultiex  - sat_basemulti )
gen tfatcost_nashsingleex_m`n' = wel_nashsingleex_m`n'/(sat_nashsingleex - sat_basesingle)
gen tfatcost_nashmultiad_m`n'  = wel_nashmultiad_m`n'/ (sat_nashmultiad  - sat_basemulti )
gen tfatcost_nashsinglead_m`n' = wel_nashsinglead_m`n'/(sat_nashsinglead - sat_basesingle)

gen cfatcost_taxmultiex_m`n'   = -CV_taxmultiex_m`n'/  (sat_taxmultiex   - sat_basemulti )
gen cfatcost_taxsingleex_m`n'  = -CV_taxsingleex_m`n'/ (sat_taxsingleex  - sat_basesingle)
gen cfatcost_taxmultiad_m`n'   = -CV_taxmultiad_m`n'/  (sat_taxmultiad   - sat_basemulti )
gen cfatcost_taxsinglead_m`n'  = -CV_taxsinglead_m`n'/ (sat_taxsinglead  - sat_basesingle)
gen cfatcost_nashmultiex_m`n'  = -CV_nashmultiex_m`n'/ (sat_nashmultiex  - sat_basemulti )
gen cfatcost_nashsingleex_m`n' = -CV_nashsingleex_m`n'/(sat_nashsingleex - sat_basesingle)
gen cfatcost_nashmultiad_m`n'  = -CV_nashmultiad_m`n'/ (sat_nashmultiad  - sat_basemulti )
gen cfatcost_nashsinglead_m`n' = -CV_nashsinglead_m`n'/(sat_nashsinglead - sat_basesingle)

}

foreach var in sat exp cost prof {
    rename `var'_basemulti    `var'_pretaxmulti
    rename `var'_basesingle   `var'_pretaxsingle
    rename `var'_taxmultiex   `var'_excisenoresmulti
    rename `var'_taxsingleex  `var'_excisenoressingle
    rename `var'_taxmultiad   `var'_advalnoresmulti
    rename `var'_taxsinglead  `var'_advalnoressingle
    rename `var'_nashmultiex  `var'_exciseresmulti
    rename `var'_nashsingleex `var'_exciseressingle
    rename `var'_nashmultiad  `var'_advalresmulti
    rename `var'_nashsinglead `var'_advalressingle
}

foreach var in tax {
    rename `var'_taxmultiex   `var'_excisenoresmulti
    rename `var'_taxsingleex  `var'_excisenoressingle
    rename `var'_taxmultiad   `var'_advalnoresmulti
    rename `var'_taxsinglead  `var'_advalnoressingle
    rename `var'_nashmultiex  `var'_exciseresmulti
    rename `var'_nashsingleex `var'_exciseressingle
    rename `var'_nashmultiad  `var'_advalresmulti
    rename `var'_nashsinglead `var'_advalressingle
}

foreach var in CV wel tfatcost cfatcost {
  foreach n of numlist 1 2 3 {
    rename `var'_taxmultiex_m`n'   `var'_m`n'_excisenoresmulti
    rename `var'_taxsingleex_m`n'  `var'_m`n'_excisenoressingle
    rename `var'_taxmultiad_m`n'   `var'_m`n'_advalnoresmulti
    rename `var'_taxsinglead_m`n'  `var'_m`n'_advalnoressingle
    rename `var'_nashmultiex_m`n'  `var'_m`n'_exciseresmulti
    rename `var'_nashsingleex_m`n' `var'_m`n'_exciseressingle
    rename `var'_nashmultiad_m`n'  `var'_m`n'_advalresmulti
    rename `var'_nashsinglead_m`n' `var'_m`n'_advalressingle
 }
}

gen i = 1
reshape long sat exp tax cost prof CV_m1 CV_m2 CV_m3 wel_m1 wel_m2 wel_m3 tfatcost_m1 tfatcost_m2 tfatcost_m3 cfatcost_m1 cfatcost_m2 cfatcost_m3,i(i) j(regime) string
drop i

foreach var in sat exp tax cost prof CV_m1 CV_m2 CV_m3 wel_m1 wel_m2 wel_m3 {
    replace `var' = `var'/1000000
}

lab var sat         "Total saturated fat (m Kg)"
lab var exp         "Total expenditure (m)"
lab var cost        "Total marginal cost (m)"
lab var prof        "Total profit (m)"
lab var CV_m1       "Total compensation variation (m) (method 1)"
lab var CV_m2       "Total compensation variation (m) (method 2)"
lab var CV_m3       "Total compensation variation (m) (method 3)"
lab var wel_m1      "Total policy cost (m) (method 1)"
lab var wel_m2      "Total policy cost (m) (method 2)"
lab var wel_m3      "Total policy cost (m) (method 3)"
lab var tfatcost_m1 "Mean cost of 1kg fat reduction () (method 1)"
lab var tfatcost_m2 "Mean cost of 1kg fat reduction () (method 2)"
lab var tfatcost_m3 "Mean cost of 1kg fat reduction () (method 3)"
lab var cfatcost_m1 "Mean consumer cost of 1kg fat reduction () (method 1)"
lab var cfatcost_m2 "Mean consumer cost of 1kg fat reduction () (method 2)"
lab var cfatcost_m3 "Mean consumer cost of 1kg fat reduction () (method 3)"
outsheet using "$DC/agg_results.csv",comma replace

sa "$DC/Aggregate_results.dta",replace

u "$DC/Household_results.dta",clear

merge 1:1 hhno using "$DC/hhweights.dta",keepusing(hhno_raw)
keep if _m==3
drop _m

rename hhno     hh
rename hhno_raw hhno

merge 1:1 hhno using "$C/net_expenditure_foodt.dta"
keep if _m==3
drop _m

su exp ,d

gen     expq = 1 if exp<r(p25)
replace expq = 2 if exp>=r(p25) & exp<r(p50)
replace expq = 3 if exp>=r(p50) & exp<r(p75)
replace expq = 4 if exp>=r(p75)

gen dsat_nashmultiex = sat_basemulti-sat_nashmultiex
gen dsat_nashmultiad = sat_basemulti-sat_nashmultiad

collapse (mean) sat_basemulti sat_nashmultiex sat_nashmultiad CV_nashmultiex_m2 CV_nashmultiad_m2 dsat_nashmultiex dsat_nashmultiad cfatcost_nashmultiex_m2 cfatcost_nashmultiad_m2,by(expq)

table expq,c(mean CV_nashmultiex_m2 mean dsat_nashmultiex mean cfatcost_nashmultiex_m2)
table expq,c(mean CV_nashmultiad_m2 mean dsat_nashmultiad mean cfatcost_nashmultiad_m2)

sa "$DC/Distributional_results.dta",replace

